import { ts } from '@ts-morph/common';
export declare function toJson(that: any, visitor: Visitor, context?: any, keys?: string[]): any;
export declare abstract class Node {
    __node: any;
    abstract visit(visitor: Visitor, context?: any): any;
    toJson(visitor: Visitor, context?: any, ...keys: any[]): any;
}
export declare type UnionOrIntersectionTypeNode = UnionTypeNode | IntersectionTypeNode;
export declare type TypeNode = KeywordTypeNode | ThisTypeNode | FunctionOrConstructorTypeNodeBase | NodeWithTypeArguments | TypePredicateNode | TypeQueryNode | TypeLiteralNode | ArrayTypeNode | TupleTypeNode | OptionalTypeNode | RestTypeNode | UnionTypeNode | IntersectionTypeNode | ConditionalTypeNode | InferTypeNode | ParenthesizedTypeNode | TypeOperatorNode | IndexedAccessTypeNode | MappedTypeNode | LiteralTypeNode | JSDocTypeExpression | JSDocType;
export declare type JSDocType = JSDocAllType | JSDocUnknownType | JSDocNonNullableType | JSDocNullableType | JSDocOptionalType | JSDocFunctionType | JSDocVariadicType | JSDocNamepathType;
export declare type JSDocTypeReferencingNode = JSDocVariadicType | JSDocOptionalType | JSDocNullableType | JSDocNonNullableType;
export declare type NodeWithTypeArguments = TypeReferenceNode;
export declare type FunctionOrConstructorTypeNodeBase = FunctionTypeNode | ConstructorTypeNode;
export declare type AccessorDeclaration = GetAccessorDeclaration | SetAccessorDeclaration;
export declare type FunctionBody = Block;
export declare type ConciseBody = FunctionBody | Expression;
export declare type NamedImportBindings = NamespaceImport | NamedImports;
export declare type Expression = OmittedExpression | UnaryExpression | UpdateExpression | PrefixUnaryExpression | YieldExpression | SyntheticExpression | ConditionalExpression | SpreadElement | AsExpression | JsxOpeningElement | JsxOpeningElement | JsxClosingFragment | JsxExpression | CommaListExpression;
export declare type ModuleName = Identifier | StringLiteral;
export declare type ModuleBody = NamespaceBody | JSDocNamespaceBody;
export declare type JSDocNamespaceBody = Identifier | JSDocNamespaceDeclaration;
export declare type NamespaceBody = ModuleBlock | NamespaceDeclaration;
export declare type UnaryExpression = UpdateExpression | DeleteExpression | TypeOfExpression | VoidExpression | AwaitExpression | TypeAssertion;
export declare type UpdateExpression = PrefixUnaryExpression | PostfixUnaryExpression | LeftHandSideExpression;
export declare type LeftHandSideExpression = MemberExpression | CallExpression | NonNullExpression | PartiallyEmittedExpression;
export declare type MemberExpression = PrimaryExpression | PropertyAccessExpression | ElementAccessExpression | TaggedTemplateExpression;
export declare type JsxOpeningLikeElement = JsxSelfClosingElement | JsxOpeningElement;
export declare type JsxAttributeLike = JsxAttribute | JsxSpreadAttribute;
export declare type JsxTagNameExpression = Identifier | ThisExpression | JsxTagNamePropertyAccess;
export declare type PrimaryExpression = Identifier | NullLiteral | BooleanLiteral | ThisExpression | SuperExpression | ImportExpression | FunctionExpression | TemplateExpression | ParenthesizedExpression | ArrayLiteralExpression | ObjectLiteralExpressionBase | NewExpression | MetaProperty | JsxElement | JsxSelfClosingElement | JsxFragment;
export declare type ObjectLiteralExpressionBase = ObjectLiteralExpression | JsxAttributes;
export declare type ObjectLiteralElementLike = PropertyAssignment | ShorthandPropertyAssignment | SpreadAssignment | MethodDeclaration | AccessorDeclaration;
export declare type Statement = NotEmittedStatement | EmptyStatement | DebuggerStatement | Block | VariableStatement | ExpressionStatement | IfStatement | IterationStatement | BreakStatement | ContinueStatement | ReturnStatement | WithStatement | SwitchStatement | LabeledStatement | ThrowStatement | TryStatement | ImportDeclaration;
export declare type IterationStatement = DoStatement | WhileStatement | ForStatement | ForInStatement | ForOfStatement;
export declare type ForInitializer = VariableDeclarationList | Expression;
export declare type BindingPattern = ObjectBindingPattern | ArrayBindingPattern;
export declare type BindingName = Identifier | BindingPattern;
export declare type ArrayBindingElement = BindingElement | OmittedExpression;
export declare type CaseOrDefaultClause = CaseClause | DefaultClause;
export declare type JsxChild = JsxText | JsxExpression | JsxElement | JsxSelfClosingElement | JsxFragment;
export declare type TemplateLiteral = TemplateExpression | NoSubstitutionTemplateLiteral;
export declare type ObjectTypeDeclaration = ClassLikeDeclaration | InterfaceDeclaration | TypeLiteralNode;
export declare type DeclarationWithTypeParameters = DeclarationWithTypeParameterChildren | JSDocTypedefTag | JSDocCallbackTag | JSDocSignature;
export declare type DeclarationWithTypeParameterChildren = SignatureDeclaration | ClassLikeDeclaration | InterfaceDeclaration | TypeAliasDeclaration | JSDocTemplateTag;
export declare type SignatureDeclaration = CallSignatureDeclaration | ConstructSignatureDeclaration | MethodSignature | IndexSignatureDeclaration | FunctionTypeNode | ConstructorTypeNode | JSDocFunctionType | FunctionDeclaration | MethodDeclaration | ConstructorDeclaration | AccessorDeclaration | FunctionExpression | ArrowFunction;
export declare type ClassLikeDeclaration = ClassDeclaration | ClassExpression;
export declare type HasJSDoc = ParameterDeclaration | CallSignatureDeclaration | ConstructSignatureDeclaration | MethodSignature | PropertySignature | ArrowFunction | ParenthesizedExpression | SpreadAssignment | ShorthandPropertyAssignment | PropertyAssignment | FunctionExpression | LabeledStatement | ExpressionStatement | VariableStatement | FunctionDeclaration | ConstructorDeclaration | MethodDeclaration | PropertyDeclaration | AccessorDeclaration | ClassLikeDeclaration | InterfaceDeclaration | TypeAliasDeclaration | EnumMember | EnumDeclaration | ModuleDeclaration | ImportEqualsDeclaration | IndexSignatureDeclaration | FunctionTypeNode | ConstructorTypeNode | JSDocFunctionType | ExportDeclaration | EndOfFileToken;
export declare type HasType = SignatureDeclaration | VariableDeclaration | ParameterDeclaration | PropertySignature | PropertyDeclaration | TypePredicateNode | ParenthesizedTypeNode | TypeOperatorNode | MappedTypeNode | AssertionExpression | TypeAliasDeclaration | JSDocTypeExpression | JSDocNonNullableType | JSDocNullableType | JSDocOptionalType | JSDocVariadicType;
export declare type HasInitializer = HasExpressionInitializer | ForStatement | ForInStatement | ForOfStatement | JsxAttribute;
export declare type HasExpressionInitializer = VariableDeclaration | ParameterDeclaration | BindingElement | PropertySignature | PropertyDeclaration | PropertyAssignment | EnumMember;
export declare type TypeElement = PropertySignature;
export declare type EntityName = Identifier | QualifiedName;
export declare type PropertyName = Identifier | StringLiteral | NumericLiteral | ComputedPropertyName;
export declare type DeclarationName = Identifier | StringLiteralLike | NumericLiteral | ComputedPropertyName | ElementAccessExpression | BindingPattern | EntityNameExpression;
export declare type StringLiteralLike = StringLiteral | NoSubstitutionTemplateLiteral;
export declare type EntityNameExpression = Identifier | PropertyAccessEntityNameExpression;
export declare type EntityNameOrEntityNameExpression = EntityName | EntityNameExpression;
export declare type ModuleReference = EntityName | ExternalModuleReference;
export declare type AssertionExpression = TypeAssertion | AsExpression;
export declare type TypeReferenceType = TypeReferenceNode | ExpressionWithTypeArguments;
export declare type ClassElement = PropertyDeclaration | SemicolonClassElement | MethodDeclaration;
export declare class Modifier extends Node {
    kind: ts.SyntaxKind.AbstractKeyword | ts.SyntaxKind.AsyncKeyword | ts.SyntaxKind.ConstKeyword | ts.SyntaxKind.DeclareKeyword | ts.SyntaxKind.DefaultKeyword | ts.SyntaxKind.ExportKeyword | ts.SyntaxKind.PublicKeyword | ts.SyntaxKind.PrivateKeyword | ts.SyntaxKind.ProtectedKeyword | ts.SyntaxKind.ReadonlyKeyword | ts.SyntaxKind.StaticKeyword;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TypeReferenceNode extends Node {
    kind: ts.SyntaxKind.TypeReference;
    typeName: EntityName;
    typeArguments?: TypeNode[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class Block extends Node {
    kind: ts.SyntaxKind.Block;
    statements: Statement[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class Identifier extends Node {
    kind: ts.SyntaxKind.Identifier;
    escapedText: string;
    originalKeywordKind?: ts.SyntaxKind;
    isInJSDocNamespace: boolean;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JSDocOptionalType extends Node {
    kind: ts.SyntaxKind.JSDocOptionalType;
    type: TypeNode;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JSDocVariadicType extends Node {
    kind: ts.SyntaxKind.JSDocVariadicType;
    type: TypeNode;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JSDocNonNullableType extends Node {
    kind: ts.SyntaxKind.JSDocNonNullableType;
    type: TypeNode;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JSDocNullableType extends Node {
    kind: ts.SyntaxKind.JSDocNullableType;
    type: TypeNode;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TypeOperatorNode extends Node {
    kind: ts.SyntaxKind.TypeOperator;
    operator: ts.SyntaxKind.KeyOfKeyword | ts.SyntaxKind.UniqueKeyword | ts.SyntaxKind.ReadonlyKeyword;
    type: TypeNode;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TypeParameterDeclaration extends Node {
    kind: ts.SyntaxKind.TypeParameter;
    name: Identifier;
    constraint?: TypeNode;
    default?: TypeNode;
    expression?: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class MappedTypeNode extends Node {
    kind: ts.SyntaxKind.MappedType;
    readonlyToken?: ts.ReadonlyToken | ts.PlusToken | ts.MinusToken;
    typeParameter: TypeParameterDeclaration;
    questionToken?: ts.QuestionToken | ts.PlusToken | ts.MinusToken;
    type?: TypeNode;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TypePredicateNode extends Node {
    kind: ts.SyntaxKind.TypePredicate;
    assertsModifier?: ts.AssertsToken;
    parameterName: Identifier | ThisTypeNode;
    type?: TypeNode;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ThisTypeNode extends Node {
    kind: ts.SyntaxKind.ThisType;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ParenthesizedTypeNode extends Node {
    kind: ts.SyntaxKind.ParenthesizedType;
    type: TypeNode;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ExternalModuleReference extends Node {
    kind: ts.SyntaxKind.ExternalModuleReference;
    expression: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class NamedExports extends Node {
    kind: ts.SyntaxKind.NamedExports;
    elements: ExportSpecifier[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class ExportDeclaration extends Node {
    kind: ts.SyntaxKind.ExportDeclaration;
    exportClause?: NamedExports;
    moduleSpecifier?: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ExportSpecifier extends Node {
    kind: ts.SyntaxKind.ExportSpecifier;
    propertyName?: Identifier;
    name: Identifier;
    visit(visitor: Visitor, context?: any): any;
}
export declare class EndOfFileToken extends Node {
    kind: ts.SyntaxKind.EndOfFileToken;
    visit(visitor: Visitor, context?: any): any;
}
export declare class EnumMember extends Node {
    kind: ts.SyntaxKind.EnumMember;
    name: PropertyName;
    initializer?: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ModuleDeclaration extends Node {
    kind: ts.SyntaxKind.ModuleDeclaration;
    name: ModuleName;
    body?: ModuleBody | JSDocNamespaceDeclaration;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ImportEqualsDeclaration extends Node {
    kind: ts.SyntaxKind.ImportEqualsDeclaration;
    name: Identifier;
    moduleReference: ModuleReference;
    visit(visitor: Visitor, context?: any): any;
}
export declare class SourceFile extends Node {
    kind: ts.SyntaxKind.SourceFile;
    statements: Statement[];
    endOfFileToken: ts.Token<ts.SyntaxKind.EndOfFileToken>;
    fileName: string;
    text: string;
    amdDependencies: ts.AmdDependency[];
    moduleName?: string;
    referencedFiles: ts.FileReference[];
    typeReferenceDirectives: ts.FileReference[];
    libReferenceDirectives: ts.FileReference[];
    languageVariant: ts.LanguageVariant;
    isDeclarationFile: boolean;
    hasNoDefaultLib: boolean;
    languageVersion: ts.ScriptTarget;
    locals: Map<string, ts.Symbol>;
    identifiers: Map<string, string>;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ConstructorDeclaration extends Node {
    kind: ts.SyntaxKind.Constructor;
    body?: FunctionBody;
    asteriskToken?: ts.AsteriskToken;
    questionToken?: ts.QuestionToken;
    exclamationToken?: ts.ExclamationToken;
    name?: PropertyName;
    typeParameters?: TypeParameterDeclaration[];
    parameters: ParameterDeclaration[];
    type?: TypeNode;
    visit(visitor: Visitor, context?: any): any;
}
export declare class FunctionDeclaration extends Node {
    kind: ts.SyntaxKind.FunctionDeclaration;
    name?: Identifier;
    body?: FunctionBody;
    asteriskToken?: ts.AsteriskToken;
    questionToken?: ts.QuestionToken;
    exclamationToken?: ts.ExclamationToken;
    typeParameters?: TypeParameterDeclaration[];
    parameters: ParameterDeclaration[];
    type?: TypeNode;
    modifiers: Modifier[];
    flags: ts.NodeFlags;
    decorators?: Decorator[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class JSDocFunctionType extends Node {
    kind: ts.SyntaxKind.JSDocFunctionType;
    visit(visitor: Visitor, context?: any): any;
}
export declare class FunctionTypeNode extends Node {
    kind: ts.SyntaxKind.FunctionType;
    type: TypeNode;
    name?: PropertyName;
    typeParameters?: TypeParameterDeclaration[];
    parameters: ParameterDeclaration[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class ConstructorTypeNode extends Node {
    kind: ts.SyntaxKind.ConstructorType;
    type: TypeNode;
    name?: PropertyName;
    typeParameters?: TypeParameterDeclaration[];
    parameters: ParameterDeclaration[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class IndexSignatureDeclaration extends Node {
    kind: ts.SyntaxKind.IndexSignature;
    locals: Map<string, ts.Symbol>;
    questionToken?: ts.QuestionToken;
    modifiers: Modifier[];
    name?: PropertyName;
    typeParameters?: TypeParameterDeclaration[];
    parameters: ParameterDeclaration[];
    type?: TypeNode;
    visit(visitor: Visitor, context?: any): any;
}
export declare class MethodSignature extends Node {
    kind: ts.SyntaxKind.MethodSignature;
    name: PropertyName;
    typeParameters?: TypeParameterDeclaration[];
    parameters: ParameterDeclaration[];
    type?: TypeNode;
    questionToken?: ts.QuestionToken;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ConstructSignatureDeclaration extends Node {
    kind: ts.SyntaxKind.ConstructSignature;
    visit(visitor: Visitor, context?: any): any;
}
export declare class KeywordTypeNode extends Node {
    kind: ts.SyntaxKind.AnyKeyword | ts.SyntaxKind.UnknownKeyword | ts.SyntaxKind.NumberKeyword | ts.SyntaxKind.BigIntKeyword | ts.SyntaxKind.ObjectKeyword | ts.SyntaxKind.BooleanKeyword | ts.SyntaxKind.StringKeyword | ts.SyntaxKind.SymbolKeyword | ts.SyntaxKind.ThisKeyword | ts.SyntaxKind.VoidKeyword | ts.SyntaxKind.UndefinedKeyword | ts.SyntaxKind.NullKeyword | ts.SyntaxKind.NeverKeyword;
    get keyword(): "object" | "symbol" | "unknown" | "any" | "string" | "number" | "undefined" | "boolean" | "bigint" | "this" | "void" | "never" | "nerver";
    visit(visitor: Visitor, context?: any): any;
}
export declare class CallSignatureDeclaration extends Node {
    kind: ts.SyntaxKind.CallSignature;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ArrowFunction extends Node {
    kind: ts.SyntaxKind.ArrowFunction;
    equalsGreaterThanToken: ts.EqualsGreaterThanToken;
    body: ConciseBody;
    name: never;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TypeAliasDeclaration extends Node {
    kind: ts.SyntaxKind.TypeAliasDeclaration;
    name: Identifier;
    typeParameters?: TypeParameterDeclaration[];
    type: TypeNode;
    visit(visitor: Visitor, context?: any): any;
}
export declare class EnumDeclaration extends Node {
    kind: ts.SyntaxKind.EnumDeclaration;
    name: Identifier;
    members: EnumMember[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class PropertyAccessEntityNameExpression extends Node {
    kind: ts.SyntaxKind.PropertyAccessExpression;
    questionDotToken?: ts.QuestionDotToken;
    name: Identifier;
    expression: EntityNameExpression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JSDocSignature extends Node {
    kind: ts.SyntaxKind.JSDocSignature;
    typeParameters?: JSDocTemplateTag[];
    parameters: JSDocParameterTag[];
    type: JSDocReturnTag | undefined;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JSDocCallbackTag extends Node {
    kind: ts.SyntaxKind.JSDocCallbackTag;
    fullName?: JSDocNamespaceDeclaration | Identifier;
    name?: Identifier;
    typeExpression: JSDocSignature;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JSDocTypeExpression extends Node {
    kind: ts.SyntaxKind.JSDocTypeExpression;
    type: TypeNode;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JSDocTemplateTag extends Node {
    kind: ts.SyntaxKind.JSDocTemplateTag;
    constraint: JSDocTypeExpression | undefined;
    typeParameters: TypeParameterDeclaration[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class JSDocParameterTag extends Node {
    kind: ts.SyntaxKind.JSDocParameterTag;
    name: EntityName;
    typeExpression?: JSDocTypeExpression;
    isNameFirst: boolean;
    isBracketed: boolean;
    tagName: Identifier;
    comment?: string;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JSDocReturnTag extends Node {
    kind: ts.SyntaxKind.JSDocReturnTag;
    typeExpression?: JSDocTypeExpression;
    tagName: Identifier;
    comment?: string;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ClassExpression extends Node {
    kind: ts.SyntaxKind.ClassExpression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class Decorator extends Node {
    kind: ts.SyntaxKind.Decorator;
    expression: LeftHandSideExpression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class QualifiedName extends Node {
    kind: ts.SyntaxKind.QualifiedName;
    left: EntityName;
    right: Identifier;
    decorators?: Decorator[];
    modifiers: Modifier[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class JSDoc extends Node {
    kind: ts.SyntaxKind.JSDocComment;
    tags?: JSDocTag[];
    comment?: string;
    visit(visitor: Visitor, context?: any): any;
}
export declare type JSDocPropertyLikeTag = JSDocPropertyTag | JSDocParameterTag;
export declare type JSDocTag = JSDocUnknownTag | JSDocAugmentsTag | JSDocAuthorTag | JSDocClassTag | JSDocEnumTag | JSDocThisTag | JSDocTemplateTag | JSDocReturnTag | JSDocTypeTag | JSDocTypedefTag | JSDocCallbackTag | JSDocPropertyLikeTag;
export declare class JSDocUnknownTag extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.JSDocTag;
}
export declare class ExpressionWithTypeArguments extends Node {
    kind: ts.SyntaxKind.ExpressionWithTypeArguments;
    expression: LeftHandSideExpression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JSDocAugmentsTag extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.JSDocAugmentsTag;
    class: ExpressionWithTypeArguments;
}
export declare class JSDocAuthorTag extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.JSDocAuthorTag;
}
export declare class JSDocClassTag extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.JSDocClassTag;
}
export declare class JSDocEnumTag extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.JSDocEnumTag;
    typeExpression?: JSDocTypeExpression;
}
export declare class JSDocThisTag extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.JSDocThisTag;
    typeExpression?: JSDocTypeExpression;
}
export declare class JSDocTypeTag extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.JSDocTypeTag;
    typeExpression: JSDocTypeExpression;
}
export declare class JSDocTypeLiteral extends Node {
    kind: ts.SyntaxKind.JSDocTypeLiteral;
    jsDocPropertyTags?: JSDocPropertyLikeTag[];
    isArrayType?: boolean;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JSDocPropertyTag extends Node {
    kind: ts.SyntaxKind.JSDocPropertyTag;
    name: EntityName;
    typeExpression?: JSDocTypeExpression;
    isNameFirst: boolean;
    isBracketed: boolean;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JSDocTypedefTag extends Node {
    kind: ts.SyntaxKind.JSDocTypedefTag;
    fullName?: JSDocNamespaceDeclaration | Identifier;
    name?: Identifier;
    typeExpression?: JSDocTypeExpression | JSDocTypeLiteral;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TypeLiteralNode extends Node {
    kind: ts.SyntaxKind.TypeLiteral;
    members: TypeElement[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class PropertySignature extends Node {
    jsDoc: JSDoc[];
    kind: ts.SyntaxKind.PropertySignature;
    name: PropertyName;
    questionToken?: ts.QuestionToken;
    type?: TypeNode;
    initializer?: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class InterfaceDeclaration extends Node {
    kind: ts.SyntaxKind.InterfaceDeclaration;
    name: Identifier;
    typeParameters?: TypeParameterDeclaration[];
    heritageClauses?: HeritageClause[];
    members: TypeElement[];
    jsDoc: JSDoc[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class NoSubstitutionTemplateLiteral extends Node {
    kind: ts.SyntaxKind.NoSubstitutionTemplateLiteral;
    rawText?: string;
    text: string;
    isUnterminated?: boolean;
    hasExtendedUnicodeEscape?: boolean;
    visit(visitor: Visitor, context?: any): any;
}
export declare class GetAccessorDeclaration extends Node {
    kind: ts.SyntaxKind.GetAccessor;
    name: PropertyName;
    body?: FunctionBody;
    visit(visitor: Visitor, context?: any): any;
}
export declare class SetAccessorDeclaration extends Node {
    kind: ts.SyntaxKind.SetAccessor;
    name: PropertyName;
    body?: FunctionBody;
    visit(visitor: Visitor, context?: any): any;
}
export declare class MethodDeclaration extends Node {
    decorators: Decorator[];
    jsDoc: JSDoc[];
    modifiers: Modifier[];
    kind: ts.SyntaxKind.MethodDeclaration;
    name: PropertyName;
    body?: FunctionBody;
    asteriskToken?: ts.AsteriskToken;
    questionToken?: ts.QuestionToken;
    exclamationToken?: ts.ExclamationToken;
    typeParameters?: TypeParameterDeclaration[];
    parameters: ParameterDeclaration[];
    type?: TypeNode;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TaggedTemplateExpression extends Node {
    kind: ts.SyntaxKind.TaggedTemplateExpression;
    tag: LeftHandSideExpression;
    typeArguments?: TypeNode[];
    template: TemplateLiteral;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JsxText extends Node {
    kind: ts.SyntaxKind.JsxText;
    containsOnlyTriviaWhiteSpaces: boolean;
    visit(visitor: Visitor, context?: any): any;
}
export declare class SpreadAssignment extends Node {
    kind: ts.SyntaxKind.SpreadAssignment;
    expression: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class PropertyAssignment extends Node {
    kind: ts.SyntaxKind.PropertyAssignment;
    name: PropertyName;
    questionToken?: ts.QuestionToken;
    initializer: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ShorthandPropertyAssignment extends Node {
    kind: ts.SyntaxKind.ShorthandPropertyAssignment;
    name: Identifier;
    questionToken?: ts.QuestionToken;
    exclamationToken?: ts.ExclamationToken;
    equalsToken?: ts.Token<ts.SyntaxKind.EqualsToken>;
    objectAssignmentInitializer?: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class EmptyStatement extends Node {
    kind: ts.SyntaxKind.EmptyStatement;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JsxTagNamePropertyAccess extends Node {
    kind: ts.SyntaxKind.PropertyAccessExpression;
    questionDotToken?: ts.QuestionDotToken;
    name: Identifier;
    expression: JsxTagNameExpression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class IfStatement extends Node {
    kind: ts.SyntaxKind.IfStatement;
    expression: Expression;
    thenStatement: Statement;
    elseStatement?: Statement;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ExpressionStatement extends Node {
    kind: ts.SyntaxKind.ExpressionStatement;
    expression: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class DebuggerStatement extends Node {
    kind: ts.SyntaxKind.DebuggerStatement;
    visit(visitor: Visitor, context?: any): any;
}
export declare class NotEmittedStatement extends Node {
    kind: ts.SyntaxKind.NotEmittedStatement;
    visit(visitor: Visitor, context?: any): any;
}
export declare class DefaultClause extends Node {
    kind: ts.SyntaxKind.DefaultClause;
    statements: Statement;
    visit(visitor: Visitor, context?: any): any;
}
export declare class NamespaceDeclaration extends Node {
    kind: ts.SyntaxKind.ModuleDeclaration;
    name: Identifier;
    body: NamespaceBody;
    visit(visitor: Visitor, context?: any): any;
}
export declare class NamedImports extends Node {
    kind: ts.SyntaxKind.NamedImports;
    elements: ImportSpecifier[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class ImportSpecifier extends Node {
    kind: ts.SyntaxKind.ImportSpecifier;
    propertyName?: Identifier;
    name: Identifier;
    visit(visitor: Visitor, context?: any): any;
}
export declare class NamespaceImport extends Node {
    kind: ts.SyntaxKind.NamespaceImport;
    name: Identifier;
    visit(visitor: Visitor, context?: any): any;
}
export declare class CaseClause extends Node {
    kind: ts.SyntaxKind.CaseClause;
    expression: Expression;
    statements: Statement;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ComputedPropertyName extends Node {
    kind: ts.SyntaxKind.ComputedPropertyName;
    expression: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ObjectBindingPattern extends Node {
    kind: ts.SyntaxKind.ObjectBindingPattern;
    elements: BindingElement[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class ArrayBindingPattern extends Node {
    kind: ts.SyntaxKind.ArrayBindingPattern;
    elements: ArrayBindingElement[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class ParameterDeclaration extends Node {
    __node: ts.ParameterDeclaration;
    kind: ts.SyntaxKind.Parameter;
    dotDotDotToken?: ts.DotDotDotToken;
    name: BindingName;
    questionToken?: ts.QuestionToken;
    type?: TypeNode;
    initializer?: Expression;
    decorators?: any;
    visit(visitor: Visitor, context?: any): any;
}
export declare class BindingElement extends Node {
    kind: ts.SyntaxKind.BindingElement;
    propertyName?: PropertyName;
    dotDotDotToken?: ts.DotDotDotToken;
    name: BindingName;
    initializer?: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class VariableDeclarationList extends Node {
    kind: ts.SyntaxKind.VariableDeclarationList;
    modifiers: Modifier[];
    flags: ts.NodeFlags;
    decorators?: Decorator[];
    declarations: VariableDeclaration[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class VariableStatement extends Node {
    kind: ts.SyntaxKind.VariableStatement;
    modifiers: Modifier[];
    flags: ts.NodeFlags;
    decorators?: Decorator[];
    declarationList: VariableDeclarationList;
    visit(visitor: Visitor, context?: any): any;
}
export declare class VariableDeclaration extends Node {
    kind: ts.SyntaxKind.VariableDeclaration;
    flags: ts.NodeFlags;
    decorators?: Decorator[];
    name: BindingName;
    exclamationToken?: ts.ExclamationToken;
    type?: TypeNode;
    initializer?: Expression;
    modifiers: Modifier[];
    visit(visitor: Visitor, context?: any): any;
    isConst(): boolean;
    isLet(): boolean;
}
export declare class DoStatement extends Node {
    kind: ts.SyntaxKind.DoStatement;
    expression: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class WhileStatement extends Node {
    kind: ts.SyntaxKind.WhileStatement;
    expression: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ForStatement extends Node {
    kind: ts.SyntaxKind.ForStatement;
    initializer?: ForInitializer;
    condition?: Expression;
    incrementor?: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ForInStatement extends Node {
    kind: ts.SyntaxKind.ForInStatement;
    initializer: ForInitializer;
    expression: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ForOfStatement extends Node {
    kind: ts.SyntaxKind.ForOfStatement;
    awaitModifier?: ts.AwaitKeywordToken;
    initializer: ForInitializer;
    expression: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class BreakStatement extends Node {
    kind: ts.SyntaxKind.BreakStatement;
    label?: Identifier;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ContinueStatement extends Node {
    kind: ts.SyntaxKind.ContinueStatement;
    label?: Identifier;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ReturnStatement extends Node {
    kind: ts.SyntaxKind.ReturnStatement;
    expression?: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class WithStatement extends Node {
    kind: ts.SyntaxKind.WithStatement;
    expression: Expression;
    statement: Statement;
    visit(visitor: Visitor, context?: any): any;
}
export declare class LabeledStatement extends Node {
    kind: ts.SyntaxKind.LabeledStatement;
    label: Identifier;
    statement: Statement;
    visit(visitor: Visitor, context?: any): any;
}
export declare class CaseBlock extends Node {
    kind: ts.SyntaxKind.CaseBlock;
    clauses: CaseOrDefaultClause;
    visit(visitor: Visitor, context?: any): any;
}
export declare class SwitchStatement extends Node {
    kind: ts.SyntaxKind.SwitchStatement;
    expression: Expression;
    caseBlock: CaseBlock;
    possiblyExhaustive?: boolean;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ThrowStatement extends Node {
    kind: ts.SyntaxKind.ThrowStatement;
    expression?: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class CatchClause extends Node {
    kind: ts.SyntaxKind.CatchClause;
    variableDeclaration?: VariableDeclaration;
    block: Block;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TryStatement extends Node {
    kind: ts.SyntaxKind.TryStatement;
    tryBlock: Block;
    catchClause?: CatchClause;
    finallyBlock?: Block;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ImportClause extends Node {
    kind: ts.SyntaxKind.ImportClause;
    name?: Identifier;
    namedBindings?: NamedImportBindings;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ImportDeclaration extends Node {
    kind: ts.SyntaxKind.ImportDeclaration;
    moduleSpecifier: Expression;
    importClause?: ImportClause;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ModuleBlock extends Node {
    kind: ts.SyntaxKind.ModuleBlock;
    statements: Statement[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class JSDocNamespaceDeclaration extends Node {
    name: Identifier;
    body?: JSDocNamespaceBody;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JsxSpreadAttribute extends Node {
    kind: ts.SyntaxKind.JsxSpreadAttribute;
    expression: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class StringLiteral extends Node {
    kind: ts.SyntaxKind.StringLiteral;
    text: string;
    isUnterminated?: boolean;
    hasExtendedUnicodeEscape?: boolean;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JsxAttribute extends Node {
    kind: ts.SyntaxKind.JsxAttribute;
    name: Identifier;
    initializer?: StringLiteral | JsxExpression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ObjectLiteralExpression extends Node {
    kind: ts.SyntaxKind.ObjectLiteralExpression;
    properties: ObjectLiteralElementLike[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class JsxAttributes extends Node {
    kind: ts.SyntaxKind.JsxAttributes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JsxOpeningFragment extends Node {
    kind: ts.SyntaxKind.JsxOpeningFragment;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JsxClosingFragment extends Node {
    kind: ts.SyntaxKind.JsxClosingFragment;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JsxFragment extends Node {
    kind: ts.SyntaxKind.JsxFragment;
    openingFragment: JsxOpeningFragment;
    children: JsxChild[];
    closingFragment: JsxClosingFragment;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JsxSelfClosingElement extends Node {
    kind: ts.SyntaxKind.JsxSelfClosingElement;
    tagName: JsxTagNameExpression;
    typeArguments?: TypeNode[];
    attributes: JsxAttributes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JsxOpeningElement extends Node {
    kind: ts.SyntaxKind.JsxOpeningElement;
    tagName: JsxTagNameExpression;
    typeArguments?: TypeNode[];
    attributes: JsxAttributes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JsxClosingElement extends Node {
    kind: ts.SyntaxKind.JsxClosingElement;
    tagName: JsxTagNameExpression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class JsxElement extends Node {
    kind: ts.SyntaxKind.JsxElement;
    openingElement: JsxOpeningElement;
    children: JsxChild[];
    closingElement: JsxClosingElement;
    visit(visitor: Visitor, context?: any): any;
}
export declare class MetaProperty extends Node {
    kind: ts.SyntaxKind.MetaProperty;
    keywordToken: ts.SyntaxKind.NewKeyword | ts.SyntaxKind.ImportKeyword;
    name: Identifier;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ArrayLiteralExpression extends Node {
    kind: ts.SyntaxKind.ArrayLiteralExpression;
    elements: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ParenthesizedExpression extends Node {
    kind: ts.SyntaxKind.ParenthesizedExpression;
    expression: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TemplateHead extends Node {
    kind: ts.SyntaxKind.TemplateHead;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TemplateExpression extends Node {
    kind: ts.SyntaxKind.TemplateExpression;
    head: TemplateHead;
    templateSpans: TemplateSpan[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class TemplateSpan extends Node {
    kind: ts.SyntaxKind.TemplateSpan;
    expression: Expression;
    literal: TemplateMiddle | TemplateTail;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TemplateMiddle extends Node {
    rawText?: string;
    kind: ts.SyntaxKind.TemplateMiddle;
    visit(visitor: Visitor, context?: any): any;
}
export declare class FunctionExpression extends Node {
    kind: ts.SyntaxKind.FunctionExpression;
    name?: Identifier;
    body: FunctionBody;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TemplateTail extends Node {
    kind: ts.SyntaxKind.TemplateTail;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ThisExpression extends Node {
    kind: ts.SyntaxKind.ThisKeyword;
    visit(visitor: Visitor, context?: any): any;
}
export declare class SuperExpression extends Node {
    kind: ts.SyntaxKind.SuperKeyword;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ImportExpression extends Node {
    kind: ts.SyntaxKind.ImportKeyword;
    visit(visitor: Visitor, context?: any): any;
}
export declare class NullLiteral extends Node {
    kind: ts.SyntaxKind.NullKeyword;
    visit(visitor: Visitor, context?: any): any;
}
export declare class BooleanLiteral extends Node {
    kind: ts.SyntaxKind.TrueKeyword | ts.SyntaxKind.FalseKeyword;
    isTrue(): boolean;
    visit(visitor: Visitor, context?: any): any;
}
export declare class PartiallyEmittedExpression extends Node {
    kind: ts.SyntaxKind.PartiallyEmittedExpression;
    expression: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class NonNullExpression extends Node {
    kind: ts.SyntaxKind.NonNullExpression;
    expression: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class PostfixUnaryExpression extends Node {
    kind: ts.SyntaxKind.PostfixUnaryExpression;
    operand: LeftHandSideExpression;
    operator: ts.PostfixUnaryOperator;
    visit(visitor: Visitor, context?: any): any;
}
export declare class PrefixUnaryExpression extends Node {
    kind: ts.SyntaxKind.PrefixUnaryExpression;
    operator: ts.PrefixUnaryOperator;
    operand: UnaryExpression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class DeleteExpression extends Node {
    kind: ts.SyntaxKind.DeleteExpression;
    expression: UnaryExpression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TypeOfExpression extends Node {
    kind: ts.SyntaxKind.TypeOfExpression;
    expression: UnaryExpression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class VoidExpression extends Node {
    kind: ts.SyntaxKind.VoidExpression;
    expression: UnaryExpression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class AwaitExpression extends Node {
    kind: ts.SyntaxKind.AwaitExpression;
    expression: UnaryExpression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TypeAssertion extends Node {
    kind: ts.SyntaxKind.TypeAssertionExpression;
    type: TypeNode;
    expression: UnaryExpression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class CommaListExpression extends Node {
    kind: ts.SyntaxKind.CommaListExpression;
    elements: Expression[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class JsxExpression extends Node {
    kind: ts.SyntaxKind.JsxExpression;
    dotDotDotToken?: ts.Token<ts.SyntaxKind.DotDotDotToken>;
    expression?: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class AsExpression extends Node {
    kind: ts.SyntaxKind.AsExpression;
    expression: Expression;
    type: TypeNode;
    visit(visitor: Visitor, context?: any): any;
}
export declare class CallExpression extends Node {
    kind: ts.SyntaxKind.CallExpression;
    expression: LeftHandSideExpression;
    questionDotToken?: ts.QuestionDotToken;
    typeArguments?: TypeNode[];
    arguments: Expression[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class NewExpression extends Node {
    kind: ts.SyntaxKind.NewExpression;
    expression: LeftHandSideExpression;
    typeArguments?: TypeNode[];
    arguments?: Expression[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class SpreadElement extends Node {
    kind: ts.SyntaxKind.SpreadElement;
    expression: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ConditionalExpression extends Node {
    kind: ts.SyntaxKind.ConditionalExpression;
    condition: Expression;
    questionToken: ts.QuestionToken;
    whenTrue: Expression;
    colonToken: ts.ColonToken;
    whenFalse: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ElementAccessExpression extends Node {
    kind: ts.SyntaxKind.ElementAccessExpression;
    expression: LeftHandSideExpression;
    questionDotToken?: ts.QuestionDotToken;
    argumentExpression: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class OmittedExpression extends Node {
    kind: ts.SyntaxKind.OmittedExpression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class YieldExpression extends Node {
    kind: ts.SyntaxKind.YieldExpression;
    asteriskToken?: ts.AsteriskToken;
    expression?: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class SyntheticExpression extends Node {
    kind: ts.SyntaxKind.SyntheticExpression;
    isSpread: boolean;
    type: ts.Type;
    visit(visitor: Visitor, context?: any): any;
}
export declare class PropertyAccessExpression extends Node {
    kind: ts.SyntaxKind.PropertyAccessExpression;
    expression: LeftHandSideExpression;
    questionDotToken?: ts.QuestionDotToken;
    name: Identifier;
    visit(visitor: Visitor, context?: any): any;
}
export declare class BigIntLiteral extends Node {
    kind: ts.SyntaxKind.BigIntLiteral;
    text: string;
    isUnterminated?: boolean;
    hasExtendedUnicodeEscape?: boolean;
    visit(visitor: Visitor, context?: any): any;
}
export declare class NumericLiteral extends Node {
    kind: ts.SyntaxKind.NumericLiteral;
    text: string;
    isUnterminated?: boolean;
    hasExtendedUnicodeEscape?: boolean;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ClassDeclaration extends Node {
    flags: ts.NodeFlags;
    decorators?: Decorator[];
    modifiers?: Modifier[];
    kind: ts.SyntaxKind.ClassDeclaration;
    name?: Identifier;
    typeParameters?: TypeParameterDeclaration[];
    heritageClauses?: HeritageClause[];
    members: ClassElement[];
    visit(visitor: Visitor, context?: any): any;
}
export declare class PropertyDeclaration extends Node {
    modifiers: Modifier[];
    flags: ts.NodeFlags;
    decorators?: Decorator[];
    jsDoc: JSDoc[];
    kind: ts.SyntaxKind.PropertyDeclaration;
    name: PropertyName;
    questionToken?: ts.QuestionToken;
    exclamationToken?: ts.ExclamationToken;
    type?: TypeNode;
    initializer?: Expression;
    visit(visitor: Visitor, context?: any): any;
}
export declare class SemicolonClassElement extends Node {
    kind: ts.SyntaxKind.SemicolonClassElement;
    visit(visitor: Visitor, context?: any): any;
}
export declare class HeritageClause extends Node {
    kind: ts.SyntaxKind.HeritageClause;
    token: ts.SyntaxKind.ExtendsKeyword | ts.SyntaxKind.ImplementsKeyword;
    types: ExpressionWithTypeArguments[];
    getToken(): "extends" | "implements";
    visit(visitor: Visitor, context?: any): any;
}
export declare class ExportAssignment extends Node {
    kind: ts.SyntaxKind.ExportAssignment;
    isExportEquals?: boolean;
    expression: Expression;
    name?: Identifier | StringLiteral | NumericLiteral;
    visit(visitor: Visitor, context?: any): any;
}
export declare type Declaration = NamedDeclaration;
export declare type NamedDeclaration = DeclarationStatement;
export declare type DeclarationStatement = FunctionDeclaration | MethodDeclaration | PropertyDeclaration;
export declare class ConditionalTypeNode extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.ConditionalType;
    checkType: TypeNode;
    extendsType: TypeNode;
    trueType: TypeNode;
    falseType: TypeNode;
}
export declare class InferTypeNode extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.InferType;
    typeParameter: TypeParameterDeclaration;
}
export declare class UnionTypeNode extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.UnionType;
    types: TypeNode[];
}
export declare class IntersectionTypeNode extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.IntersectionType;
    types: TypeNode[];
}
export declare class IndexedAccessTypeNode extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.IndexedAccessType;
    objectType: TypeNode;
    indexType: TypeNode;
}
export declare type LiteralExpression = RegularExpressionLiteral | NoSubstitutionTemplateLiteral | NumericLiteral | BigIntLiteral;
export declare class RegularExpressionLiteral extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.RegularExpressionLiteral;
    text: string;
    isUnterminated?: boolean;
    hasExtendedUnicodeEscape?: boolean;
}
export declare class LiteralTypeNode extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.LiteralType;
    literal: BooleanLiteral | LiteralExpression | PrefixUnaryExpression;
}
export declare class TypeQueryNode extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.TypeQuery;
    exprName: EntityName;
}
export declare class JSDocAllType extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.JSDocAllType;
}
export declare class JSDocUnknownType extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.JSDocUnknownType;
}
export declare class JSDocNamepathType extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.JSDocNamepathType;
    type: TypeNode;
}
export declare class ArrayTypeNode extends Node {
    __node: ts.ArrayTypeNode;
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.ArrayType;
    elementType: TypeNode;
}
export declare class TupleTypeNode extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.TupleType;
    elementTypes: TypeNode[];
}
export declare class OptionalTypeNode extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.OptionalType;
    type: TypeNode;
}
export declare class RestTypeNode extends Node {
    visit(visitor: Visitor, context?: any): any;
    kind: ts.SyntaxKind.RestType;
    type: TypeNode;
}
export declare type Symbol = FunctionScopedVariableSymbol | BlockScopedVariableSymbol;
export declare class FunctionScopedVariableSymbol extends Node {
    flags: ts.SymbolFlags.FunctionScopedVariable;
    intrinsicName: string;
    visit(visitor: Visitor, context?: any): any;
}
export declare class BlockScopedVariableSymbol extends Node {
    flags: ts.SymbolFlags.BlockScopedVariable;
    visit(visitor: Visitor, context?: any): any;
}
export declare class PropertySymbol extends Node {
    flags: ts.SymbolFlags.Property;
    name: string;
    escapedName: string;
    valueDeclaration: PropertySignature;
    intrinsicName: string;
    visit(visitor: Visitor, context?: any): any;
}
export declare class PropertyType {
}
export declare class EnumMemberSymbol extends Node {
    flags: ts.SymbolFlags.EnumMember;
    name: string;
    escapedName: string;
    valueDeclaration: Node;
    visit(visitor: Visitor, context?: any): any;
}
export declare class FunctionSymbol extends Node {
    declarations: FunctionDeclaration[];
    flags: ts.SymbolFlags.Function;
    id: number;
    name: string;
    valueDeclaration: FunctionDeclaration;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ClassSymbol extends Node {
    __node: ts.Symbol;
    declarations: ClassDeclaration[];
    flags: ts.SymbolFlags.Class;
    escapedName: string;
    name: string;
    id: number;
    members: Map<string, ts.Symbol>;
    valueDeclaration: ClassDeclaration;
    visit(visitor: Visitor, context?: any): any;
}
export declare class InterfaceSymbol extends Node {
    declarations: InterfaceDeclaration[];
    flags: ts.SymbolFlags.Interface;
    id: number;
    name: string;
    escapedName: string;
    members: Map<string, ts.Symbol>;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ConstEnumSymbol extends Node {
    flags: ts.SymbolFlags.ConstEnum;
    visit(visitor: Visitor, context?: any): any;
}
export declare class RegularEnumSymbol extends Node {
    flags: ts.SymbolFlags.RegularEnum;
    escapedName: string;
    name: string;
    valueDeclaration: EnumDeclaration;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ValueModuleSymbol extends Node {
    flags: ts.SymbolFlags.ValueModule;
    visit(visitor: Visitor, context?: any): any;
}
export declare class NamespaceModuleSymbol extends Node {
    flags: ts.SymbolFlags.NamespaceModule;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TypeLiteralSymbol extends Node {
    flags: ts.SymbolFlags.TypeLiteral;
    members: Map<string, ts.Symbol>;
    name: string;
    escapedName: string;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ObjectLiteralSymbol extends Node {
    flags: ts.SymbolFlags.ObjectLiteral;
    visit(visitor: Visitor, context?: any): any;
}
export declare class MethodSymbol extends Node {
    flags: ts.SymbolFlags.Method;
    name: string;
    escapedName: string;
    valueDeclaration: MethodSignature;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ConstructorSymbol extends Node {
    flags: ts.SymbolFlags.Constructor;
    visit(visitor: Visitor, context?: any): any;
}
export declare class GetAccessorSymbol extends Node {
    flags: ts.SymbolFlags.GetAccessor;
    visit(visitor: Visitor, context?: any): any;
}
export declare class SetAccessorSymbol extends Node {
    flags: ts.SymbolFlags.SetAccessor;
    visit(visitor: Visitor, context?: any): any;
}
export declare class SignatureSymbol extends Node {
    flags: ts.SymbolFlags.Signature;
    name: string;
    escapedName: string;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TypeParameterSymbol extends Node {
    flags: ts.SymbolFlags.TypeParameter;
    name: string;
    escapedName: string;
    declarations: Node[];
    id: number;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TypeAliasSymbol extends Node {
    declarations: Node[];
    flags: ts.SymbolFlags.TypeAlias;
    escapedName: string;
    name: string;
    members: Map<string, any>;
    numberIndexInfo: any;
    stringIndexInfo: any;
    symbol: any;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ExportValueSymbol extends Node {
    flags: ts.SymbolFlags.ExportValue;
    visit(visitor: Visitor, context?: any): any;
}
export declare class AliasSymbol extends Node {
    flags: ts.SymbolFlags.Alias;
    declarations: Node[];
    escapedName: string;
    name: string;
    visit(visitor: Visitor, context?: any): any;
    get(checker: ts.TypeChecker): ts.Type;
}
export declare class PrototypeSymbol extends Node {
    flags: ts.SymbolFlags.Prototype;
    visit(visitor: Visitor, context?: any): any;
}
export declare class OptionalSymbol extends Node {
    flags: ts.SymbolFlags.Optional;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TransientSymbol extends Node {
    flags: ts.SymbolFlags.Transient;
    visit(visitor: Visitor, context?: any): any;
}
export declare class AssignmentSymbol extends Node {
    flags: ts.SymbolFlags.Assignment;
    intrinsicName: string;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ModuleExportsSymbol extends Node {
    flags: ts.SymbolFlags.ModuleExports;
    visit(visitor: Visitor, context?: any): any;
}
export declare class EnumSymbol extends Node {
    flags: ts.SymbolFlags.Enum;
    visit(visitor: Visitor, context?: any): any;
}
export declare class VariableSymbol extends Node {
    flags: ts.SymbolFlags.Variable;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ValueSymbol extends Node {
    flags: ts.SymbolFlags.Value;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TypeSymbol extends Node {
    flags: ts.SymbolFlags.Type;
    visit(visitor: Visitor, context?: any): any;
}
export declare class NamespaceSymbol extends Node {
    flags: ts.SymbolFlags.Namespace;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ModuleSymbol extends Node {
    flags: ts.SymbolFlags.Module;
    visit(visitor: Visitor, context?: any): any;
}
export declare class AccessorSymbol extends Node {
    flags: ts.SymbolFlags.Accessor;
    visit(visitor: Visitor, context?: any): any;
}
export declare class FunctionScopedVariableExcludesSymbol extends Node {
    flags: ts.SymbolFlags.FunctionScopedVariableExcludes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class BlockScopedVariableExcludesSymbol extends Node {
    flags: ts.SymbolFlags.BlockScopedVariableExcludes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ParameterExcludesSymbol extends Node {
    flags: ts.SymbolFlags.ParameterExcludes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class EnumMemberExcludesSymbol extends Node {
    flags: ts.SymbolFlags.EnumMemberExcludes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class FunctionExcludesSymbol extends Node {
    flags: ts.SymbolFlags.FunctionExcludes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ClassExcludesSymbol extends Node {
    flags: ts.SymbolFlags.ClassExcludes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class InterfaceExcludesSymbol extends Node {
    flags: ts.SymbolFlags.InterfaceExcludes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class RegularEnumExcludesSymbol extends Node {
    flags: ts.SymbolFlags.RegularEnumExcludes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ConstEnumExcludesSymbol extends Node {
    flags: ts.SymbolFlags.ConstEnumExcludes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ValueModuleExcludesSymbol extends Node {
    flags: ts.SymbolFlags.ValueModuleExcludes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class MethodExcludesSymbol extends Node {
    flags: ts.SymbolFlags.MethodExcludes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class GetAccessorExcludesSymbol extends Node {
    flags: ts.SymbolFlags.GetAccessorExcludes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class SetAccessorExcludesSymbol extends Node {
    flags: ts.SymbolFlags.SetAccessorExcludes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TypeParameterExcludesSymbol extends Node {
    flags: ts.SymbolFlags.TypeParameterExcludes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class TypeAliasExcludesSymbol extends Node {
    flags: ts.SymbolFlags.TypeAliasExcludes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class AliasExcludesSymbol extends Node {
    flags: ts.SymbolFlags.AliasExcludes;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ModuleMemberSymbol extends Node {
    flags: ts.SymbolFlags.ModuleMember;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ExportHasLocalSymbol extends Node {
    flags: ts.SymbolFlags.ExportHasLocal;
    visit(visitor: Visitor, context?: any): any;
}
export declare class BlockScopedSymbol extends Node {
    flags: ts.SymbolFlags.BlockScoped;
    visit(visitor: Visitor, context?: any): any;
}
export declare class PropertyOrAccessorSymbol extends Node {
    flags: ts.SymbolFlags.PropertyOrAccessor;
    visit(visitor: Visitor, context?: any): any;
}
export declare class ClassMemberSymbol extends Node {
    flags: ts.SymbolFlags.ClassMember;
    visit(visitor: Visitor, context?: any): any;
}
export declare type DestructuringPattern = BindingPattern | ObjectLiteralExpression | ArrayLiteralExpression;
export declare class Type {
    id: number;
    flags: ts.TypeFlags;
    symbol: Symbol;
    pattern?: DestructuringPattern;
    aliasSymbol?: Symbol;
    aliasTypeArguments?: Type[];
}
export declare class ObjectType {
}
export declare class TypeReference {
}
export interface Visitor {
    visitFunctionScopedVariableSymbol(node: FunctionScopedVariableSymbol, context?: any): any;
    visitBlockScopedVariableSymbol(node: BlockScopedVariableSymbol, context?: any): any;
    visitPropertySymbol(node: PropertySymbol, context?: any): any;
    visitEnumMemberSymbol(node: EnumMemberSymbol, context?: any): any;
    visitFunctionSymbol(node: FunctionSymbol, context?: any): any;
    visitClassSymbol(node: ClassSymbol, context?: any): any;
    visitInterfaceSymbol(node: InterfaceSymbol, context?: any): any;
    visitConstEnumSymbol(node: ConstEnumSymbol, context?: any): any;
    visitRegularEnumSymbol(node: RegularEnumSymbol, context?: any): any;
    visitValueModuleSymbol(node: ValueModuleSymbol, context?: any): any;
    visitNamespaceModuleSymbol(node: NamespaceModuleSymbol, context?: any): any;
    visitTypeLiteralSymbol(node: TypeLiteralSymbol, context?: any): any;
    visitObjectLiteralSymbol(node: ObjectLiteralSymbol, context?: any): any;
    visitMethodSymbol(node: MethodSymbol, context?: any): any;
    visitConstructorSymbol(node: ConstructorSymbol, context?: any): any;
    visitGetAccessorSymbol(node: GetAccessorSymbol, context?: any): any;
    visitSetAccessorSymbol(node: SetAccessorSymbol, context?: any): any;
    visitSignatureSymbol(node: SignatureSymbol, context?: any): any;
    visitTypeParameterSymbol(node: TypeParameterSymbol, context?: any): any;
    visitTypeAliasSymbol(node: TypeAliasSymbol, context?: any): any;
    visitExportValueSymbol(node: ExportValueSymbol, context?: any): any;
    visitAliasSymbol(node: AliasSymbol, context?: any): any;
    visitPrototypeSymbol(node: PrototypeSymbol, context?: any): any;
    visitOptionalSymbol(node: OptionalSymbol, context?: any): any;
    visitTransientSymbol(node: TransientSymbol, context?: any): any;
    visitAssignmentSymbol(node: AssignmentSymbol, context?: any): any;
    visitModuleExportsSymbol(node: ModuleExportsSymbol, context?: any): any;
    visitEnumSymbol(node: EnumSymbol, context?: any): any;
    visitVariableSymbol(node: VariableSymbol, context?: any): any;
    visitValueSymbol(node: ValueSymbol, context?: any): any;
    visitTypeSymbol(node: TypeSymbol, context?: any): any;
    visitNamespaceSymbol(node: NamespaceSymbol, context?: any): any;
    visitModuleSymbol(node: ModuleSymbol, context?: any): any;
    visitAccessorSymbol(node: AccessorSymbol, context?: any): any;
    visitFunctionScopedVariableExcludesSymbol(node: FunctionScopedVariableExcludesSymbol, context?: any): any;
    visitBlockScopedVariableExcludesSymbol(node: BlockScopedVariableExcludesSymbol, context?: any): any;
    visitParameterExcludesSymbol(node: ParameterExcludesSymbol, context?: any): any;
    visitEnumMemberExcludesSymbol(node: EnumMemberExcludesSymbol, context?: any): any;
    visitFunctionExcludesSymbol(node: FunctionExcludesSymbol, context?: any): any;
    visitClassExcludesSymbol(node: ClassExcludesSymbol, context?: any): any;
    visitInterfaceExcludesSymbol(node: InterfaceExcludesSymbol, context?: any): any;
    visitRegularEnumExcludesSymbol(node: RegularEnumExcludesSymbol, context?: any): any;
    visitConstEnumExcludesSymbol(node: ConstEnumExcludesSymbol, context?: any): any;
    visitValueModuleExcludesSymbol(node: ValueModuleExcludesSymbol, context?: any): any;
    visitMethodExcludesSymbol(node: MethodExcludesSymbol, context?: any): any;
    visitGetAccessorExcludesSymbol(node: GetAccessorExcludesSymbol, context?: any): any;
    visitSetAccessorExcludesSymbol(node: SetAccessorExcludesSymbol, context?: any): any;
    visitTypeParameterExcludesSymbol(node: TypeParameterExcludesSymbol, context?: any): any;
    visitTypeAliasExcludesSymbol(node: TypeAliasExcludesSymbol, context?: any): any;
    visitAliasExcludesSymbol(node: AliasExcludesSymbol, context?: any): any;
    visitModuleMemberSymbol(node: ModuleMemberSymbol, context?: any): any;
    visitExportHasLocalSymbol(node: ExportHasLocalSymbol, context?: any): any;
    visitBlockScopedSymbol(node: BlockScopedSymbol, context?: any): any;
    visitPropertyOrAccessorSymbol(node: PropertyOrAccessorSymbol, context?: any): any;
    visitClassMemberSymbol(node: ClassMemberSymbol, context?: any): any;
    visitConditionalTypeNode(node: ConditionalTypeNode, context: any): any;
    visitInferTypeNode(node: InferTypeNode, context: any): any;
    visitUnionTypeNode(node: UnionTypeNode, context: any): any;
    visitIntersectionTypeNode(node: IntersectionTypeNode, context: any): any;
    visitIndexedAccessTypeNode(node: IndexedAccessTypeNode, context: any): any;
    visitRegularExpressionLiteral(node: RegularExpressionLiteral, context: any): any;
    visitLiteralTypeNode(node: LiteralTypeNode, context: any): any;
    visitTypeQueryNode(node: TypeQueryNode, context: any): any;
    visitJSDocAllType(node: JSDocAllType, context: any): any;
    visitJSDocUnknownType(node: JSDocUnknownType, context: any): any;
    visitJSDocNamepathType(node: JSDocNamepathType, context: any): any;
    visitArrayTypeNode(node: ArrayTypeNode, context: any): any;
    visitTupleTypeNode(node: TupleTypeNode, context: any): any;
    visitOptionalTypeNode(node: OptionalTypeNode, context: any): any;
    visitRestTypeNode(node: RestTypeNode, context: any): any;
    visitModifier(node: Modifier, context?: any): any;
    visitJSDocThisTag(node: JSDocThisTag, context?: any): any;
    visitJSDocTypeTag(node: JSDocTypeTag, context?: any): any;
    visitJSDocUnknownTag(node: JSDocUnknownTag, context?: any): any;
    visitJSDocAugmentsTag(node: JSDocAugmentsTag, context?: any): any;
    visitJSDocAuthorTag(node: JSDocAuthorTag, context?: any): any;
    visitJSDocClassTag(node: JSDocClassTag, context?: any): any;
    visitJSDocEnumTag(node: JSDocEnumTag, context?: any): any;
    visitParenthesizedTypeNode(node: ParenthesizedTypeNode, context?: any): any;
    visitSetAccessorDeclaration(node: SetAccessorDeclaration, context?: any): any;
    visitMethodDeclaration(node: MethodDeclaration, context?: any): any;
    visitTaggedTemplateExpression(node: TaggedTemplateExpression, context?: any): any;
    visitJsxText(node: JsxText, context?: any): any;
    visitSpreadAssignment(node: SpreadAssignment, context?: any): any;
    visitPropertyAssignment(node: PropertyAssignment, context?: any): any;
    visitShorthandPropertyAssignment(node: ShorthandPropertyAssignment, context?: any): any;
    visitEmptyStatement(node: EmptyStatement, context?: any): any;
    visitJsxTagNamePropertyAccess(node: JsxTagNamePropertyAccess, context?: any): any;
    visitIfStatement(node: IfStatement, context?: any): any;
    visitDebuggerStatement(node: DebuggerStatement, context?: any): any;
    visitNotEmittedStatement(node: NotEmittedStatement, context?: any): any;
    visitDefaultClause(node: DefaultClause, context?: any): any;
    visitNamespaceDeclaration(node: NamespaceDeclaration, context?: any): any;
    visitNamedImports(node: NamedImports, context?: any): any;
    visitImportSpecifier(node: ImportSpecifier, context?: any): any;
    visitNamespaceImport(node: NamespaceImport, context?: any): any;
    visitCaseClause(node: CaseClause, context?: any): any;
    visitComputedPropertyName(node: ComputedPropertyName, context?: any): any;
    visitObjectBindingPattern(node: ObjectBindingPattern, context?: any): any;
    visitArrayBindingPattern(node: ArrayBindingPattern, context?: any): any;
    visitParameterDeclaration(node: ParameterDeclaration, context?: any): any;
    visitBindingElement(node: BindingElement, context?: any): any;
    visitVariableDeclarationList(node: VariableDeclarationList, context?: any): any;
    visitVariableStatement(node: VariableStatement, context?: any): any;
    visitVariableDeclaration(node: VariableDeclaration, context?: any): any;
    visitDoStatement(node: DoStatement, context?: any): any;
    visitWhileStatement(node: WhileStatement, context?: any): any;
    visitForStatement(node: ForStatement, context?: any): any;
    visitForInStatement(node: ForInStatement, context?: any): any;
    visitForOfStatement(node: ForOfStatement, context?: any): any;
    visitBreakStatement(node: BreakStatement, context?: any): any;
    visitContinueStatement(node: ContinueStatement, context?: any): any;
    visitReturnStatement(node: ReturnStatement, context?: any): any;
    visitWithStatement(node: WithStatement, context?: any): any;
    visitLabeledStatement(node: LabeledStatement, context?: any): any;
    visitCaseBlock(node: CaseBlock, context?: any): any;
    visitSwitchStatement(node: SwitchStatement, context?: any): any;
    visitThrowStatement(node: ThrowStatement, context?: any): any;
    visitCatchClause(node: CatchClause, context?: any): any;
    visitTryStatement(node: TryStatement, context?: any): any;
    visitImportClause(node: ImportClause, context?: any): any;
    visitImportDeclaration(node: ImportDeclaration, context?: any): any;
    visitModuleBlock(node: ModuleBlock, context?: any): any;
    visitJSDocNamespaceDeclaration(node: JSDocNamespaceDeclaration, context?: any): any;
    visitJsxSpreadAttribute(node: JsxSpreadAttribute, context?: any): any;
    visitStringLiteral(node: StringLiteral, context?: any): any;
    visitJsxAttribute(node: JsxAttribute, context?: any): any;
    visitObjectLiteralExpression(node: ObjectLiteralExpression, context?: any): any;
    visitJsxAttributes(node: JsxAttributes, context?: any): any;
    visitJsxOpeningFragment(node: JsxOpeningFragment, context?: any): any;
    visitJsxClosingFragment(node: JsxClosingFragment, context?: any): any;
    visitJsxFragment(node: JsxFragment, context?: any): any;
    visitJsxSelfClosingElement(node: JsxSelfClosingElement, context?: any): any;
    visitJsxOpeningElement(node: JsxOpeningElement, context?: any): any;
    visitJsxClosingElement(node: JsxClosingElement, context?: any): any;
    visitJsxElement(node: JsxElement, context?: any): any;
    visitMetaProperty(node: MetaProperty, context?: any): any;
    visitArrayLiteralExpression(node: ArrayLiteralExpression, context?: any): any;
    visitParenthesizedExpression(node: ParenthesizedExpression, context?: any): any;
    visitTemplateHead(node: TemplateHead, context?: any): any;
    visitTemplateExpression(node: TemplateExpression, context?: any): any;
    visitTemplateSpan(node: TemplateSpan, context?: any): any;
    visitTemplateMiddle(node: TemplateMiddle, context?: any): any;
    visitFunctionExpression(node: FunctionExpression, context?: any): any;
    visitTemplateTail(node: TemplateTail, context?: any): any;
    visitThisExpression(node: ThisExpression, context?: any): any;
    visitSuperExpression(node: SuperExpression, context?: any): any;
    visitImportExpression(node: ImportExpression, context?: any): any;
    visitNullLiteral(node: NullLiteral, context?: any): any;
    visitBooleanLiteral(node: BooleanLiteral, context?: any): any;
    visitPartiallyEmittedExpression(node: PartiallyEmittedExpression, context?: any): any;
    visitNonNullExpression(node: NonNullExpression, context?: any): any;
    visitPostfixUnaryExpression(node: PostfixUnaryExpression, context?: any): any;
    visitPrefixUnaryExpression(node: PrefixUnaryExpression, context?: any): any;
    visitDeleteExpression(node: DeleteExpression, context?: any): any;
    visitTypeOfExpression(node: TypeOfExpression, context?: any): any;
    visitVoidExpression(node: VoidExpression, context?: any): any;
    visitAwaitExpression(node: AwaitExpression, context?: any): any;
    visitTypeAssertion(node: TypeAssertion, context?: any): any;
    visitCommaListExpression(node: CommaListExpression, context?: any): any;
    visitJsxExpression(node: JsxExpression, context?: any): any;
    visitAsExpression(node: AsExpression, context?: any): any;
    visitNewExpression(node: NewExpression, context?: any): any;
    visitSpreadElement(node: SpreadElement, context?: any): any;
    visitConditionalExpression(node: ConditionalExpression, context?: any): any;
    visitElementAccessExpression(node: ElementAccessExpression, context?: any): any;
    visitOmittedExpression(node: OmittedExpression, context?: any): any;
    visitYieldExpression(node: YieldExpression, context?: any): any;
    visitSyntheticExpression(node: SyntheticExpression, context?: any): any;
    visitPropertyAccessExpression(node: PropertyAccessExpression, context?: any): any;
    visitBigIntLiteral(node: BigIntLiteral, context?: any): any;
    visitNumericLiteral(node: NumericLiteral, context?: any): any;
    visitSemicolonClassElement(node: SemicolonClassElement, context?: any): any;
    visitHeritageClause(node: HeritageClause, context?: any): any;
    visitExpressionWithTypeArguments(node: ExpressionWithTypeArguments, context?: any): any;
    visitJSDocVariadicType(node: JSDocVariadicType, context?: any): any;
    visitJSDocNonNullableType(node: JSDocNonNullableType, context?: any): any;
    visitJSDocNullableType(node: JSDocNullableType, context?: any): any;
    visitTypeOperatorNode(node: TypeOperatorNode, context?: any): any;
    visitTypeParameterDeclaration(node: TypeParameterDeclaration, context?: any): any;
    visitMappedTypeNode(node: MappedTypeNode, context?: any): any;
    visitTypePredicateNode(node: TypePredicateNode, context?: any): any;
    visitThisTypeNode(node: ThisTypeNode, context?: any): any;
    visitExternalModuleReference(node: ExternalModuleReference, context?: any): any;
    visitNamedExports(node: NamedExports, context?: any): any;
    visitExportDeclaration(node: ExportDeclaration, context?: any): any;
    visitExportSpecifier(node: ExportSpecifier, context?: any): any;
    visitEndOfFileToken(node: EndOfFileToken, context?: any): any;
    visitEnumMember(node: EnumMember, context?: any): any;
    visitModuleDeclaration(node: ModuleDeclaration, context?: any): any;
    visitImportEqualsDeclaration(node: ImportEqualsDeclaration, context?: any): any;
    visitFunctionDeclaration(node: FunctionDeclaration, context?: any): any;
    visitJSDocFunctionType(node: JSDocFunctionType, context?: any): any;
    visitFunctionTypeNode(node: FunctionTypeNode, context?: any): any;
    visitConstructorTypeNode(node: ConstructorTypeNode, context: any): any;
    visitIndexSignatureDeclaration(node: IndexSignatureDeclaration, context?: any): any;
    visitMethodSignature(node: MethodSignature, context?: any): any;
    visitConstructSignatureDeclaration(node: ConstructSignatureDeclaration, context?: any): any;
    visitCallSignatureDeclaration(node: CallSignatureDeclaration, context?: any): any;
    visitArrowFunction(node: ArrowFunction, context?: any): any;
    visitTypeAliasDeclaration(node: TypeAliasDeclaration, context?: any): any;
    visitEnumDeclaration(node: EnumDeclaration, context?: any): any;
    visitPropertyAccessEntityNameExpression(node: PropertyAccessEntityNameExpression, context?: any): any;
    visitJSDocSignature(node: JSDocSignature, context?: any): any;
    visitJSDocCallbackTag(node: JSDocCallbackTag, context?: any): any;
    visitJSDocTypeExpression(node: JSDocTypeExpression, context?: any): any;
    visitJSDocTemplateTag(node: JSDocTemplateTag, context?: any): any;
    visitJSDocParameterTag(node: JSDocParameterTag, context?: any): any;
    visitJSDocReturnTag(node: JSDocReturnTag, context?: any): any;
    visitClassExpression(node: ClassExpression, context?: any): any;
    visitDecorator(node: Decorator, context?: any): any;
    visitJSDoc(node: JSDoc, context?: any): any;
    visitJSDocTypeLiteral(node: JSDocTypeLiteral, context?: any): any;
    visitJSDocPropertyTag(node: JSDocPropertyTag, context?: any): any;
    visitJSDocTypedefTag(node: JSDocTypedefTag, context?: any): any;
    visitTypeLiteralNode(node: TypeLiteralNode, context?: any): any;
    visitPropertySignature(node: PropertySignature, context?: any): any;
    visitInterfaceDeclaration(node: InterfaceDeclaration, context?: any): any;
    visitNoSubstitutionTemplateLiteral(node: NoSubstitutionTemplateLiteral, context?: any): any;
    visitGetAccessorDeclaration(node: GetAccessorDeclaration, context?: any): any;
    visit(node?: Node, context?: any): any;
    visits(nodes?: Node[], context?: any): any;
    visitSourceFile(node: SourceFile, context?: any): any;
    visitClassDeclaration(node: ClassDeclaration, context?: any): any;
    visitPropertyDeclaration(node: PropertyDeclaration, context?: any): any;
    visitTypeReferenceNode(node: TypeReferenceNode, context?: any): any;
    visitQualifiedName(node: QualifiedName, context?: any): any;
    visitIdentifier(node: Identifier, context?: any): any;
    visitKeywordTypeNode(node: KeywordTypeNode, context?: any): any;
    visitConstructorDeclaration(node: ConstructorDeclaration, context?: any): any;
    visitExpressionStatement(node: ExpressionStatement, context?: any): any;
    visitCallExpression(node: CallExpression, context?: any): any;
    visitBlock(node: Block, context?: any): any;
    visitJSDocOptionalType(node: JSDocOptionalType, context?: any): any;
    visitExportAssignment(node: ExportAssignment, context?: any): any;
}
