"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const common_1 = require("@ts-morph/common");
const core_1 = require("@notadd/core");
function toJson(that, visitor, context, keys = []) {
    return core_1.toPlain(that, 'kind', (source, instance) => {
        if (keys && keys.includes(source.property)) {
            return;
        }
        if (source && source.options && source.options.isClass) {
            if (Array.isArray(instance)) {
                return visitor.visits(instance, context);
            }
            return visitor.visit(instance, context);
        }
        return instance;
    });
}
exports.toJson = toJson;
class Node {
    toJson(visitor, context, ...keys) {
        return toJson(this, visitor, context, keys);
    }
}
exports.Node = Node;
let Modifier = class Modifier extends Node {
    visit(visitor, context) {
        return visitor.visitModifier(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], Modifier.prototype, "kind", void 0);
Modifier = tslib_1.__decorate([
    core_1.Plain({
        desc: [
            common_1.ts.SyntaxKind.AbstractKeyword,
            common_1.ts.SyntaxKind.AsyncKeyword,
            common_1.ts.SyntaxKind.ConstKeyword,
            common_1.ts.SyntaxKind.DeclareKeyword,
            common_1.ts.SyntaxKind.DefaultKeyword,
            common_1.ts.SyntaxKind.ExportKeyword,
            common_1.ts.SyntaxKind.PublicKeyword,
            common_1.ts.SyntaxKind.PrivateKeyword,
            common_1.ts.SyntaxKind.ProtectedKeyword,
            common_1.ts.SyntaxKind.ReadonlyKeyword,
            common_1.ts.SyntaxKind.StaticKeyword
        ]
    })
], Modifier);
exports.Modifier = Modifier;
let TypeReferenceNode = class TypeReferenceNode extends Node {
    visit(visitor, context) {
        return visitor.visitTypeReferenceNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypeReferenceNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], TypeReferenceNode.prototype, "typeName", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], TypeReferenceNode.prototype, "typeArguments", void 0);
TypeReferenceNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TypeReference
    })
], TypeReferenceNode);
exports.TypeReferenceNode = TypeReferenceNode;
let Block = class Block extends Node {
    visit(visitor, context) {
        return visitor.visitBlock(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], Block.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], Block.prototype, "statements", void 0);
Block = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.Block
    })
], Block);
exports.Block = Block;
let Identifier = class Identifier extends Node {
    visit(visitor, context) {
        return visitor.visitIdentifier(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], Identifier.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], Identifier.prototype, "escapedText", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], Identifier.prototype, "originalKeywordKind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], Identifier.prototype, "isInJSDocNamespace", void 0);
Identifier = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.Identifier
    })
], Identifier);
exports.Identifier = Identifier;
let JSDocOptionalType = class JSDocOptionalType extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocOptionalType(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocOptionalType.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], JSDocOptionalType.prototype, "type", void 0);
JSDocOptionalType = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocOptionalType
    })
], JSDocOptionalType);
exports.JSDocOptionalType = JSDocOptionalType;
let JSDocVariadicType = class JSDocVariadicType extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocVariadicType(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocVariadicType.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], JSDocVariadicType.prototype, "type", void 0);
JSDocVariadicType = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocVariadicType
    })
], JSDocVariadicType);
exports.JSDocVariadicType = JSDocVariadicType;
let JSDocNonNullableType = class JSDocNonNullableType extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocNonNullableType(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocNonNullableType.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], JSDocNonNullableType.prototype, "type", void 0);
JSDocNonNullableType = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocNonNullableType
    })
], JSDocNonNullableType);
exports.JSDocNonNullableType = JSDocNonNullableType;
let JSDocNullableType = class JSDocNullableType extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocNullableType(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocNullableType.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], JSDocNullableType.prototype, "type", void 0);
JSDocNullableType = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocNullableType
    })
], JSDocNullableType);
exports.JSDocNullableType = JSDocNullableType;
let TypeOperatorNode = class TypeOperatorNode extends Node {
    visit(visitor, context) {
        return visitor.visitTypeOperatorNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypeOperatorNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypeOperatorNode.prototype, "operator", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], TypeOperatorNode.prototype, "type", void 0);
TypeOperatorNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TypeOperator
    })
], TypeOperatorNode);
exports.TypeOperatorNode = TypeOperatorNode;
let TypeParameterDeclaration = class TypeParameterDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitTypeParameterDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypeParameterDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], TypeParameterDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], TypeParameterDeclaration.prototype, "constraint", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], TypeParameterDeclaration.prototype, "default", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], TypeParameterDeclaration.prototype, "expression", void 0);
TypeParameterDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TypeParameter
    })
], TypeParameterDeclaration);
exports.TypeParameterDeclaration = TypeParameterDeclaration;
let MappedTypeNode = class MappedTypeNode extends Node {
    visit(visitor, context) {
        return visitor.visitMappedTypeNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], MappedTypeNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], MappedTypeNode.prototype, "readonlyToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", TypeParameterDeclaration)
], MappedTypeNode.prototype, "typeParameter", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], MappedTypeNode.prototype, "questionToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], MappedTypeNode.prototype, "type", void 0);
MappedTypeNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.MappedType
    })
], MappedTypeNode);
exports.MappedTypeNode = MappedTypeNode;
let TypePredicateNode = class TypePredicateNode extends Node {
    visit(visitor, context) {
        return visitor.visitTypePredicateNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypePredicateNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], TypePredicateNode.prototype, "assertsModifier", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], TypePredicateNode.prototype, "parameterName", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], TypePredicateNode.prototype, "type", void 0);
TypePredicateNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TypePredicate
    })
], TypePredicateNode);
exports.TypePredicateNode = TypePredicateNode;
let ThisTypeNode = class ThisTypeNode extends Node {
    visit(visitor, context) {
        return visitor.visitThisTypeNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ThisTypeNode.prototype, "kind", void 0);
ThisTypeNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ThisType
    })
], ThisTypeNode);
exports.ThisTypeNode = ThisTypeNode;
let ParenthesizedTypeNode = class ParenthesizedTypeNode extends Node {
    visit(visitor, context) {
        return visitor.visitParenthesizedTypeNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ParenthesizedTypeNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ParenthesizedTypeNode.prototype, "type", void 0);
ParenthesizedTypeNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ParenthesizedType
    })
], ParenthesizedTypeNode);
exports.ParenthesizedTypeNode = ParenthesizedTypeNode;
let ExternalModuleReference = class ExternalModuleReference extends Node {
    visit(visitor, context) {
        return visitor.visitExternalModuleReference(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ExternalModuleReference.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ExternalModuleReference.prototype, "expression", void 0);
ExternalModuleReference = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ExternalModuleReference
    })
], ExternalModuleReference);
exports.ExternalModuleReference = ExternalModuleReference;
let NamedExports = class NamedExports extends Node {
    visit(visitor, context) {
        return visitor.visitNamedExports(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], NamedExports.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], NamedExports.prototype, "elements", void 0);
NamedExports = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.NamedExports
    })
], NamedExports);
exports.NamedExports = NamedExports;
let ExportDeclaration = class ExportDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitExportDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ExportDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", NamedExports)
], ExportDeclaration.prototype, "exportClause", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ExportDeclaration.prototype, "moduleSpecifier", void 0);
ExportDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ExportDeclaration
    })
], ExportDeclaration);
exports.ExportDeclaration = ExportDeclaration;
let ExportSpecifier = class ExportSpecifier extends Node {
    visit(visitor, context) {
        return visitor.visitExportSpecifier(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ExportSpecifier.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], ExportSpecifier.prototype, "propertyName", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], ExportSpecifier.prototype, "name", void 0);
ExportSpecifier = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ExportSpecifier
    })
], ExportSpecifier);
exports.ExportSpecifier = ExportSpecifier;
let EndOfFileToken = class EndOfFileToken extends Node {
    visit(visitor, context) {
        return visitor.visitEndOfFileToken(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], EndOfFileToken.prototype, "kind", void 0);
EndOfFileToken = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.EndOfFileToken
    })
], EndOfFileToken);
exports.EndOfFileToken = EndOfFileToken;
let EnumMember = class EnumMember extends Node {
    visit(visitor, context) {
        return visitor.visitEnumMember(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], EnumMember.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], EnumMember.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], EnumMember.prototype, "initializer", void 0);
EnumMember = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.EnumMember
    })
], EnumMember);
exports.EnumMember = EnumMember;
let ModuleDeclaration = class ModuleDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitModuleDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ModuleDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ModuleDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ModuleDeclaration.prototype, "body", void 0);
ModuleDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ModuleDeclaration
    })
], ModuleDeclaration);
exports.ModuleDeclaration = ModuleDeclaration;
let ImportEqualsDeclaration = class ImportEqualsDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitImportEqualsDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ImportEqualsDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Identifier)
], ImportEqualsDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ImportEqualsDeclaration.prototype, "moduleReference", void 0);
ImportEqualsDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ImportEqualsDeclaration
    })
], ImportEqualsDeclaration);
exports.ImportEqualsDeclaration = ImportEqualsDeclaration;
let SourceFile = class SourceFile extends Node {
    visit(visitor, context) {
        return visitor.visitSourceFile(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], SourceFile.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], SourceFile.prototype, "statements", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], SourceFile.prototype, "endOfFileToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], SourceFile.prototype, "fileName", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], SourceFile.prototype, "text", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Array)
], SourceFile.prototype, "amdDependencies", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], SourceFile.prototype, "moduleName", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Array)
], SourceFile.prototype, "referencedFiles", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Array)
], SourceFile.prototype, "typeReferenceDirectives", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Array)
], SourceFile.prototype, "libReferenceDirectives", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], SourceFile.prototype, "languageVariant", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], SourceFile.prototype, "isDeclarationFile", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], SourceFile.prototype, "hasNoDefaultLib", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], SourceFile.prototype, "languageVersion", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Map)
], SourceFile.prototype, "locals", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Map)
], SourceFile.prototype, "identifiers", void 0);
SourceFile = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.SourceFile
    })
], SourceFile);
exports.SourceFile = SourceFile;
let ConstructorDeclaration = class ConstructorDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitConstructorDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ConstructorDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ConstructorDeclaration.prototype, "body", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], ConstructorDeclaration.prototype, "asteriskToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], ConstructorDeclaration.prototype, "questionToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], ConstructorDeclaration.prototype, "exclamationToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ConstructorDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], ConstructorDeclaration.prototype, "typeParameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], ConstructorDeclaration.prototype, "parameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ConstructorDeclaration.prototype, "type", void 0);
ConstructorDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.Constructor
    })
], ConstructorDeclaration);
exports.ConstructorDeclaration = ConstructorDeclaration;
let FunctionDeclaration = class FunctionDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitFunctionDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], FunctionDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], FunctionDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], FunctionDeclaration.prototype, "body", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], FunctionDeclaration.prototype, "asteriskToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], FunctionDeclaration.prototype, "questionToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], FunctionDeclaration.prototype, "exclamationToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], FunctionDeclaration.prototype, "typeParameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], FunctionDeclaration.prototype, "parameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], FunctionDeclaration.prototype, "type", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], FunctionDeclaration.prototype, "modifiers", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], FunctionDeclaration.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], FunctionDeclaration.prototype, "decorators", void 0);
FunctionDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.FunctionDeclaration
    })
], FunctionDeclaration);
exports.FunctionDeclaration = FunctionDeclaration;
let JSDocFunctionType = class JSDocFunctionType extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocFunctionType(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocFunctionType.prototype, "kind", void 0);
JSDocFunctionType = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocFunctionType
    })
], JSDocFunctionType);
exports.JSDocFunctionType = JSDocFunctionType;
let FunctionTypeNode = class FunctionTypeNode extends Node {
    visit(visitor, context) {
        return visitor.visitFunctionTypeNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], FunctionTypeNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], FunctionTypeNode.prototype, "type", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], FunctionTypeNode.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], FunctionTypeNode.prototype, "typeParameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], FunctionTypeNode.prototype, "parameters", void 0);
FunctionTypeNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.FunctionType
    })
], FunctionTypeNode);
exports.FunctionTypeNode = FunctionTypeNode;
let ConstructorTypeNode = class ConstructorTypeNode extends Node {
    visit(visitor, context) {
        return visitor.visitConstructorTypeNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ConstructorTypeNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ConstructorTypeNode.prototype, "type", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ConstructorTypeNode.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], ConstructorTypeNode.prototype, "typeParameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], ConstructorTypeNode.prototype, "parameters", void 0);
ConstructorTypeNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ConstructorType
    })
], ConstructorTypeNode);
exports.ConstructorTypeNode = ConstructorTypeNode;
let IndexSignatureDeclaration = class IndexSignatureDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitIndexSignatureDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], IndexSignatureDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Map)
], IndexSignatureDeclaration.prototype, "locals", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], IndexSignatureDeclaration.prototype, "questionToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], IndexSignatureDeclaration.prototype, "modifiers", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], IndexSignatureDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], IndexSignatureDeclaration.prototype, "typeParameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], IndexSignatureDeclaration.prototype, "parameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], IndexSignatureDeclaration.prototype, "type", void 0);
IndexSignatureDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.IndexSignature
    })
], IndexSignatureDeclaration);
exports.IndexSignatureDeclaration = IndexSignatureDeclaration;
let MethodSignature = class MethodSignature extends Node {
    visit(visitor, context) {
        return visitor.visitMethodSignature(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], MethodSignature.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], MethodSignature.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], MethodSignature.prototype, "typeParameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], MethodSignature.prototype, "parameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], MethodSignature.prototype, "type", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], MethodSignature.prototype, "questionToken", void 0);
MethodSignature = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.MethodSignature
    })
], MethodSignature);
exports.MethodSignature = MethodSignature;
let ConstructSignatureDeclaration = class ConstructSignatureDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitConstructSignatureDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ConstructSignatureDeclaration.prototype, "kind", void 0);
ConstructSignatureDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ConstructSignature
    })
], ConstructSignatureDeclaration);
exports.ConstructSignatureDeclaration = ConstructSignatureDeclaration;
let KeywordTypeNode = class KeywordTypeNode extends Node {
    get keyword() {
        switch (this.kind) {
            case common_1.ts.SyntaxKind.AnyKeyword:
                return 'any';
            case common_1.ts.SyntaxKind.UnknownKeyword:
                return 'unknown';
            case common_1.ts.SyntaxKind.NumberKeyword:
                return 'number';
            case common_1.ts.SyntaxKind.BigIntKeyword:
                return 'bigint';
            case common_1.ts.SyntaxKind.ObjectKeyword:
                return 'object';
            case common_1.ts.SyntaxKind.BooleanKeyword:
                return 'boolean';
            case common_1.ts.SyntaxKind.SymbolKeyword:
                return 'symbol';
            case common_1.ts.SyntaxKind.ThisKeyword:
                return 'this';
            case common_1.ts.SyntaxKind.VoidKeyword:
                return 'void';
            case common_1.ts.SyntaxKind.UndefinedKeyword:
                return 'undefined';
            case common_1.ts.SyntaxKind.StringKeyword:
                return 'string';
            case common_1.ts.SyntaxKind.NeverKeyword:
                return 'never';
            default:
                return 'nerver';
        }
    }
    visit(visitor, context) {
        return visitor.visitKeywordTypeNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], KeywordTypeNode.prototype, "kind", void 0);
KeywordTypeNode = tslib_1.__decorate([
    core_1.Plain({
        desc: [
            common_1.ts.SyntaxKind.AnyKeyword, common_1.ts.SyntaxKind.UnknownKeyword,
            common_1.ts.SyntaxKind.NumberKeyword, common_1.ts.SyntaxKind.BigIntKeyword,
            common_1.ts.SyntaxKind.ObjectKeyword, common_1.ts.SyntaxKind.BooleanKeyword,
            common_1.ts.SyntaxKind.StringKeyword, common_1.ts.SyntaxKind.SymbolKeyword,
            common_1.ts.SyntaxKind.AnyKeyword, common_1.ts.SyntaxKind.UnknownKeyword,
            common_1.ts.SyntaxKind.NumberKeyword, common_1.ts.SyntaxKind.BigIntKeyword,
            common_1.ts.SyntaxKind.ObjectKeyword, common_1.ts.SyntaxKind.BooleanKeyword,
            common_1.ts.SyntaxKind.StringKeyword, common_1.ts.SyntaxKind.SymbolKeyword,
            common_1.ts.SyntaxKind.ThisKeyword, common_1.ts.SyntaxKind.VoidKeyword,
            common_1.ts.SyntaxKind.UndefinedKeyword, common_1.ts.SyntaxKind.NullKeyword,
            common_1.ts.SyntaxKind.NeverKeyword
        ]
    })
], KeywordTypeNode);
exports.KeywordTypeNode = KeywordTypeNode;
let CallSignatureDeclaration = class CallSignatureDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitCallSignatureDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], CallSignatureDeclaration.prototype, "kind", void 0);
CallSignatureDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.CallSignature
    })
], CallSignatureDeclaration);
exports.CallSignatureDeclaration = CallSignatureDeclaration;
let ArrowFunction = class ArrowFunction extends Node {
    visit(visitor, context) {
        return visitor.visitArrowFunction(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ArrowFunction.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], ArrowFunction.prototype, "equalsGreaterThanToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ArrowFunction.prototype, "body", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", void 0)
], ArrowFunction.prototype, "name", void 0);
ArrowFunction = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ArrowFunction
    })
], ArrowFunction);
exports.ArrowFunction = ArrowFunction;
let TypeAliasDeclaration = class TypeAliasDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitTypeAliasDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypeAliasDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], TypeAliasDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], TypeAliasDeclaration.prototype, "typeParameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], TypeAliasDeclaration.prototype, "type", void 0);
TypeAliasDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TypeAliasDeclaration
    })
], TypeAliasDeclaration);
exports.TypeAliasDeclaration = TypeAliasDeclaration;
let EnumDeclaration = class EnumDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitEnumDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], EnumDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], EnumDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], EnumDeclaration.prototype, "members", void 0);
EnumDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.EnumDeclaration
    })
], EnumDeclaration);
exports.EnumDeclaration = EnumDeclaration;
let PropertyAccessEntityNameExpression = class PropertyAccessEntityNameExpression extends Node {
    visit(visitor, context) {
        return visitor.visitPropertyAccessEntityNameExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], PropertyAccessEntityNameExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], PropertyAccessEntityNameExpression.prototype, "questionDotToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], PropertyAccessEntityNameExpression.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], PropertyAccessEntityNameExpression.prototype, "expression", void 0);
PropertyAccessEntityNameExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.PropertyAccessExpression
    })
], PropertyAccessEntityNameExpression);
exports.PropertyAccessEntityNameExpression = PropertyAccessEntityNameExpression;
let JSDocSignature = class JSDocSignature extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocSignature(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocSignature.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], JSDocSignature.prototype, "typeParameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], JSDocSignature.prototype, "parameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], JSDocSignature.prototype, "type", void 0);
JSDocSignature = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocSignature
    })
], JSDocSignature);
exports.JSDocSignature = JSDocSignature;
let JSDocCallbackTag = class JSDocCallbackTag extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocCallbackTag(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocCallbackTag.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], JSDocCallbackTag.prototype, "fullName", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], JSDocCallbackTag.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", JSDocSignature)
], JSDocCallbackTag.prototype, "typeExpression", void 0);
JSDocCallbackTag = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocCallbackTag
    })
], JSDocCallbackTag);
exports.JSDocCallbackTag = JSDocCallbackTag;
let JSDocTypeExpression = class JSDocTypeExpression extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocTypeExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocTypeExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], JSDocTypeExpression.prototype, "type", void 0);
JSDocTypeExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocTypeExpression
    })
], JSDocTypeExpression);
exports.JSDocTypeExpression = JSDocTypeExpression;
let JSDocTemplateTag = class JSDocTemplateTag extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocTemplateTag(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocTemplateTag.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], JSDocTemplateTag.prototype, "constraint", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], JSDocTemplateTag.prototype, "typeParameters", void 0);
JSDocTemplateTag = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocTemplateTag
    })
], JSDocTemplateTag);
exports.JSDocTemplateTag = JSDocTemplateTag;
let JSDocParameterTag = class JSDocParameterTag extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocParameterTag(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocParameterTag.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], JSDocParameterTag.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", JSDocTypeExpression)
], JSDocParameterTag.prototype, "typeExpression", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], JSDocParameterTag.prototype, "isNameFirst", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], JSDocParameterTag.prototype, "isBracketed", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], JSDocParameterTag.prototype, "tagName", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], JSDocParameterTag.prototype, "comment", void 0);
JSDocParameterTag = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocParameterTag
    })
], JSDocParameterTag);
exports.JSDocParameterTag = JSDocParameterTag;
let JSDocReturnTag = class JSDocReturnTag extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocReturnTag(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocReturnTag.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", JSDocTypeExpression)
], JSDocReturnTag.prototype, "typeExpression", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], JSDocReturnTag.prototype, "tagName", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], JSDocReturnTag.prototype, "comment", void 0);
JSDocReturnTag = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocReturnTag
    })
], JSDocReturnTag);
exports.JSDocReturnTag = JSDocReturnTag;
let ClassExpression = class ClassExpression extends Node {
    visit(visitor, context) {
        return visitor.visitClassExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ClassExpression.prototype, "kind", void 0);
ClassExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ClassExpression
    })
], ClassExpression);
exports.ClassExpression = ClassExpression;
let Decorator = class Decorator extends Node {
    visit(visitor, context) {
        return visitor.visitDecorator(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], Decorator.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], Decorator.prototype, "expression", void 0);
Decorator = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.Decorator
    })
], Decorator);
exports.Decorator = Decorator;
let QualifiedName = class QualifiedName extends Node {
    visit(visitor, context) {
        return visitor.visitQualifiedName(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], QualifiedName.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], QualifiedName.prototype, "left", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], QualifiedName.prototype, "right", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], QualifiedName.prototype, "decorators", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Array)
], QualifiedName.prototype, "modifiers", void 0);
QualifiedName = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.QualifiedName
    })
], QualifiedName);
exports.QualifiedName = QualifiedName;
let JSDoc = class JSDoc extends Node {
    visit(visitor, context) {
        return visitor.visitJSDoc(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDoc.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], JSDoc.prototype, "tags", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], JSDoc.prototype, "comment", void 0);
JSDoc = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocComment
    })
], JSDoc);
exports.JSDoc = JSDoc;
let JSDocUnknownTag = class JSDocUnknownTag extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocUnknownTag(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocUnknownTag.prototype, "kind", void 0);
JSDocUnknownTag = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocTag
    })
], JSDocUnknownTag);
exports.JSDocUnknownTag = JSDocUnknownTag;
let ExpressionWithTypeArguments = class ExpressionWithTypeArguments extends Node {
    visit(visitor, context) {
        return visitor.visitExpressionWithTypeArguments(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ExpressionWithTypeArguments.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ExpressionWithTypeArguments.prototype, "expression", void 0);
ExpressionWithTypeArguments = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ExpressionWithTypeArguments
    })
], ExpressionWithTypeArguments);
exports.ExpressionWithTypeArguments = ExpressionWithTypeArguments;
let JSDocAugmentsTag = class JSDocAugmentsTag extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocAugmentsTag(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocAugmentsTag.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", ExpressionWithTypeArguments)
], JSDocAugmentsTag.prototype, "class", void 0);
JSDocAugmentsTag = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocAugmentsTag
    })
], JSDocAugmentsTag);
exports.JSDocAugmentsTag = JSDocAugmentsTag;
let JSDocAuthorTag = class JSDocAuthorTag extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocAuthorTag(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocAuthorTag.prototype, "kind", void 0);
JSDocAuthorTag = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocAuthorTag
    })
], JSDocAuthorTag);
exports.JSDocAuthorTag = JSDocAuthorTag;
let JSDocClassTag = class JSDocClassTag extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocClassTag(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocClassTag.prototype, "kind", void 0);
JSDocClassTag = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocClassTag
    })
], JSDocClassTag);
exports.JSDocClassTag = JSDocClassTag;
let JSDocEnumTag = class JSDocEnumTag extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocEnumTag(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocEnumTag.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", JSDocTypeExpression)
], JSDocEnumTag.prototype, "typeExpression", void 0);
JSDocEnumTag = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocEnumTag
    })
], JSDocEnumTag);
exports.JSDocEnumTag = JSDocEnumTag;
let JSDocThisTag = class JSDocThisTag extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocThisTag(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocThisTag.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", JSDocTypeExpression)
], JSDocThisTag.prototype, "typeExpression", void 0);
JSDocThisTag = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocThisTag
    })
], JSDocThisTag);
exports.JSDocThisTag = JSDocThisTag;
let JSDocTypeTag = class JSDocTypeTag extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocTypeTag(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocTypeTag.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", JSDocTypeExpression)
], JSDocTypeTag.prototype, "typeExpression", void 0);
JSDocTypeTag = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocTypeTag
    })
], JSDocTypeTag);
exports.JSDocTypeTag = JSDocTypeTag;
let JSDocTypeLiteral = class JSDocTypeLiteral extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocTypeLiteral(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocTypeLiteral.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], JSDocTypeLiteral.prototype, "jsDocPropertyTags", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], JSDocTypeLiteral.prototype, "isArrayType", void 0);
JSDocTypeLiteral = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocTypeLiteral
    })
], JSDocTypeLiteral);
exports.JSDocTypeLiteral = JSDocTypeLiteral;
let JSDocPropertyTag = class JSDocPropertyTag extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocPropertyTag(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocPropertyTag.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], JSDocPropertyTag.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", JSDocTypeExpression)
], JSDocPropertyTag.prototype, "typeExpression", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], JSDocPropertyTag.prototype, "isNameFirst", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], JSDocPropertyTag.prototype, "isBracketed", void 0);
JSDocPropertyTag = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocPropertyTag
    })
], JSDocPropertyTag);
exports.JSDocPropertyTag = JSDocPropertyTag;
let JSDocTypedefTag = class JSDocTypedefTag extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocTypedefTag(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocTypedefTag.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], JSDocTypedefTag.prototype, "fullName", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Identifier)
], JSDocTypedefTag.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], JSDocTypedefTag.prototype, "typeExpression", void 0);
JSDocTypedefTag = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocTypedefTag
    })
], JSDocTypedefTag);
exports.JSDocTypedefTag = JSDocTypedefTag;
let TypeLiteralNode = class TypeLiteralNode extends Node {
    visit(visitor, context) {
        return visitor.visitTypeLiteralNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypeLiteralNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], TypeLiteralNode.prototype, "members", void 0);
TypeLiteralNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TypeLiteral
    })
], TypeLiteralNode);
exports.TypeLiteralNode = TypeLiteralNode;
let PropertySignature = class PropertySignature extends Node {
    visit(visitor, context) {
        return visitor.visitPropertySignature(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], PropertySignature.prototype, "jsDoc", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], PropertySignature.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], PropertySignature.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], PropertySignature.prototype, "questionToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], PropertySignature.prototype, "type", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], PropertySignature.prototype, "initializer", void 0);
PropertySignature = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.PropertySignature
    })
], PropertySignature);
exports.PropertySignature = PropertySignature;
let InterfaceDeclaration = class InterfaceDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitInterfaceDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], InterfaceDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Identifier)
], InterfaceDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], InterfaceDeclaration.prototype, "typeParameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], InterfaceDeclaration.prototype, "heritageClauses", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], InterfaceDeclaration.prototype, "members", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], InterfaceDeclaration.prototype, "jsDoc", void 0);
InterfaceDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.InterfaceDeclaration
    })
], InterfaceDeclaration);
exports.InterfaceDeclaration = InterfaceDeclaration;
let NoSubstitutionTemplateLiteral = class NoSubstitutionTemplateLiteral extends Node {
    visit(visitor, context) {
        return visitor.visitNoSubstitutionTemplateLiteral(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], NoSubstitutionTemplateLiteral.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], NoSubstitutionTemplateLiteral.prototype, "rawText", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], NoSubstitutionTemplateLiteral.prototype, "text", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], NoSubstitutionTemplateLiteral.prototype, "isUnterminated", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], NoSubstitutionTemplateLiteral.prototype, "hasExtendedUnicodeEscape", void 0);
NoSubstitutionTemplateLiteral = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.NoSubstitutionTemplateLiteral
    })
], NoSubstitutionTemplateLiteral);
exports.NoSubstitutionTemplateLiteral = NoSubstitutionTemplateLiteral;
let GetAccessorDeclaration = class GetAccessorDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitGetAccessorDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], GetAccessorDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], GetAccessorDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], GetAccessorDeclaration.prototype, "body", void 0);
GetAccessorDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.GetAccessor
    })
], GetAccessorDeclaration);
exports.GetAccessorDeclaration = GetAccessorDeclaration;
let SetAccessorDeclaration = class SetAccessorDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitSetAccessorDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], SetAccessorDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], SetAccessorDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], SetAccessorDeclaration.prototype, "body", void 0);
SetAccessorDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.SetAccessor
    })
], SetAccessorDeclaration);
exports.SetAccessorDeclaration = SetAccessorDeclaration;
let MethodDeclaration = class MethodDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitMethodDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], MethodDeclaration.prototype, "decorators", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], MethodDeclaration.prototype, "jsDoc", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], MethodDeclaration.prototype, "modifiers", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], MethodDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], MethodDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], MethodDeclaration.prototype, "body", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], MethodDeclaration.prototype, "asteriskToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], MethodDeclaration.prototype, "questionToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], MethodDeclaration.prototype, "exclamationToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], MethodDeclaration.prototype, "typeParameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], MethodDeclaration.prototype, "parameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], MethodDeclaration.prototype, "type", void 0);
MethodDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.MethodDeclaration
    })
], MethodDeclaration);
exports.MethodDeclaration = MethodDeclaration;
let TaggedTemplateExpression = class TaggedTemplateExpression extends Node {
    visit(visitor, context) {
        return visitor.visitTaggedTemplateExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TaggedTemplateExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], TaggedTemplateExpression.prototype, "tag", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], TaggedTemplateExpression.prototype, "typeArguments", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], TaggedTemplateExpression.prototype, "template", void 0);
TaggedTemplateExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TaggedTemplateExpression
    })
], TaggedTemplateExpression);
exports.TaggedTemplateExpression = TaggedTemplateExpression;
let JsxText = class JsxText extends Node {
    visit(visitor, context) {
        return visitor.visitJsxText(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JsxText.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], JsxText.prototype, "containsOnlyTriviaWhiteSpaces", void 0);
JsxText = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JsxText
    })
], JsxText);
exports.JsxText = JsxText;
let SpreadAssignment = class SpreadAssignment extends Node {
    visit(visitor, context) {
        return visitor.visitSpreadAssignment(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], SpreadAssignment.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], SpreadAssignment.prototype, "expression", void 0);
SpreadAssignment = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.SpreadAssignment
    })
], SpreadAssignment);
exports.SpreadAssignment = SpreadAssignment;
let PropertyAssignment = class PropertyAssignment extends Node {
    visit(visitor, context) {
        return visitor.visitPropertyAssignment(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], PropertyAssignment.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], PropertyAssignment.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], PropertyAssignment.prototype, "questionToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], PropertyAssignment.prototype, "initializer", void 0);
PropertyAssignment = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.PropertyAssignment
    })
], PropertyAssignment);
exports.PropertyAssignment = PropertyAssignment;
let ShorthandPropertyAssignment = class ShorthandPropertyAssignment extends Node {
    visit(visitor, context) {
        return visitor.visitShorthandPropertyAssignment(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ShorthandPropertyAssignment.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], ShorthandPropertyAssignment.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], ShorthandPropertyAssignment.prototype, "questionToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], ShorthandPropertyAssignment.prototype, "exclamationToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], ShorthandPropertyAssignment.prototype, "equalsToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ShorthandPropertyAssignment.prototype, "objectAssignmentInitializer", void 0);
ShorthandPropertyAssignment = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ShorthandPropertyAssignment
    })
], ShorthandPropertyAssignment);
exports.ShorthandPropertyAssignment = ShorthandPropertyAssignment;
let EmptyStatement = class EmptyStatement extends Node {
    visit(visitor, context) {
        return visitor.visitEmptyStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], EmptyStatement.prototype, "kind", void 0);
EmptyStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.EmptyStatement
    })
], EmptyStatement);
exports.EmptyStatement = EmptyStatement;
let JsxTagNamePropertyAccess = class JsxTagNamePropertyAccess extends Node {
    visit(visitor, context) {
        return visitor.visitJsxTagNamePropertyAccess(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JsxTagNamePropertyAccess.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], JsxTagNamePropertyAccess.prototype, "questionDotToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], JsxTagNamePropertyAccess.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], JsxTagNamePropertyAccess.prototype, "expression", void 0);
JsxTagNamePropertyAccess = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.PropertyAccessExpression
    })
], JsxTagNamePropertyAccess);
exports.JsxTagNamePropertyAccess = JsxTagNamePropertyAccess;
let IfStatement = class IfStatement extends Node {
    visit(visitor, context) {
        return visitor.visitIfStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], IfStatement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], IfStatement.prototype, "expression", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], IfStatement.prototype, "thenStatement", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], IfStatement.prototype, "elseStatement", void 0);
IfStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.IfStatement
    })
], IfStatement);
exports.IfStatement = IfStatement;
let ExpressionStatement = class ExpressionStatement extends Node {
    visit(visitor, context) {
        return visitor.visitExpressionStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ExpressionStatement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ExpressionStatement.prototype, "expression", void 0);
ExpressionStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ExpressionStatement
    })
], ExpressionStatement);
exports.ExpressionStatement = ExpressionStatement;
let DebuggerStatement = class DebuggerStatement extends Node {
    visit(visitor, context) {
        return visitor.visitDebuggerStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], DebuggerStatement.prototype, "kind", void 0);
DebuggerStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.DebuggerStatement
    })
], DebuggerStatement);
exports.DebuggerStatement = DebuggerStatement;
let NotEmittedStatement = class NotEmittedStatement extends Node {
    visit(visitor, context) {
        return visitor.visitNotEmittedStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], NotEmittedStatement.prototype, "kind", void 0);
NotEmittedStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.NotEmittedStatement
    })
], NotEmittedStatement);
exports.NotEmittedStatement = NotEmittedStatement;
let DefaultClause = class DefaultClause extends Node {
    visit(visitor, context) {
        return visitor.visitDefaultClause(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], DefaultClause.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], DefaultClause.prototype, "statements", void 0);
DefaultClause = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.DefaultClause
    })
], DefaultClause);
exports.DefaultClause = DefaultClause;
let NamespaceDeclaration = class NamespaceDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitNamespaceDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], NamespaceDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Identifier)
], NamespaceDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], NamespaceDeclaration.prototype, "body", void 0);
NamespaceDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ModuleDeclaration
    })
], NamespaceDeclaration);
exports.NamespaceDeclaration = NamespaceDeclaration;
let NamedImports = class NamedImports extends Node {
    visit(visitor, context) {
        return visitor.visitNamedImports(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], NamedImports.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], NamedImports.prototype, "elements", void 0);
NamedImports = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.NamedImports
    })
], NamedImports);
exports.NamedImports = NamedImports;
let ImportSpecifier = class ImportSpecifier extends Node {
    visit(visitor, context) {
        return visitor.visitImportSpecifier(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ImportSpecifier.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], ImportSpecifier.prototype, "propertyName", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], ImportSpecifier.prototype, "name", void 0);
ImportSpecifier = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ImportSpecifier
    })
], ImportSpecifier);
exports.ImportSpecifier = ImportSpecifier;
let NamespaceImport = class NamespaceImport extends Node {
    visit(visitor, context) {
        return visitor.visitNamespaceImport(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], NamespaceImport.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], NamespaceImport.prototype, "name", void 0);
NamespaceImport = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.NamespaceImport
    })
], NamespaceImport);
exports.NamespaceImport = NamespaceImport;
let CaseClause = class CaseClause extends Node {
    visit(visitor, context) {
        return visitor.visitCaseClause(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], CaseClause.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], CaseClause.prototype, "expression", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], CaseClause.prototype, "statements", void 0);
CaseClause = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.CaseClause
    })
], CaseClause);
exports.CaseClause = CaseClause;
let ComputedPropertyName = class ComputedPropertyName extends Node {
    visit(visitor, context) {
        return visitor.visitComputedPropertyName(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ComputedPropertyName.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ComputedPropertyName.prototype, "expression", void 0);
ComputedPropertyName = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ComputedPropertyName
    })
], ComputedPropertyName);
exports.ComputedPropertyName = ComputedPropertyName;
let ObjectBindingPattern = class ObjectBindingPattern extends Node {
    visit(visitor, context) {
        return visitor.visitObjectBindingPattern(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ObjectBindingPattern.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], ObjectBindingPattern.prototype, "elements", void 0);
ObjectBindingPattern = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ObjectBindingPattern
    })
], ObjectBindingPattern);
exports.ObjectBindingPattern = ObjectBindingPattern;
let ArrayBindingPattern = class ArrayBindingPattern extends Node {
    visit(visitor, context) {
        return visitor.visitArrayBindingPattern(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ArrayBindingPattern.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], ArrayBindingPattern.prototype, "elements", void 0);
ArrayBindingPattern = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ArrayBindingPattern
    })
], ArrayBindingPattern);
exports.ArrayBindingPattern = ArrayBindingPattern;
let ParameterDeclaration = class ParameterDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitParameterDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ParameterDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], ParameterDeclaration.prototype, "dotDotDotToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ParameterDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], ParameterDeclaration.prototype, "questionToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ParameterDeclaration.prototype, "type", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ParameterDeclaration.prototype, "initializer", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ParameterDeclaration.prototype, "decorators", void 0);
ParameterDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.Parameter
    })
], ParameterDeclaration);
exports.ParameterDeclaration = ParameterDeclaration;
let BindingElement = class BindingElement extends Node {
    visit(visitor, context) {
        return visitor.visitBindingElement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], BindingElement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], BindingElement.prototype, "propertyName", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], BindingElement.prototype, "dotDotDotToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], BindingElement.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], BindingElement.prototype, "initializer", void 0);
BindingElement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.BindingElement
    })
], BindingElement);
exports.BindingElement = BindingElement;
let VariableDeclarationList = class VariableDeclarationList extends Node {
    visit(visitor, context) {
        return visitor.visitVariableDeclarationList(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], VariableDeclarationList.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Array)
], VariableDeclarationList.prototype, "modifiers", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], VariableDeclarationList.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], VariableDeclarationList.prototype, "decorators", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], VariableDeclarationList.prototype, "declarations", void 0);
VariableDeclarationList = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.VariableDeclarationList
    })
], VariableDeclarationList);
exports.VariableDeclarationList = VariableDeclarationList;
let VariableStatement = class VariableStatement extends Node {
    visit(visitor, context) {
        return visitor.visitVariableStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], VariableStatement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Array)
], VariableStatement.prototype, "modifiers", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], VariableStatement.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], VariableStatement.prototype, "decorators", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", VariableDeclarationList)
], VariableStatement.prototype, "declarationList", void 0);
VariableStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.VariableStatement
    })
], VariableStatement);
exports.VariableStatement = VariableStatement;
let VariableDeclaration = class VariableDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitVariableDeclaration(this, context);
    }
    isConst() {
        return this.flags === common_1.ts.NodeFlags.Const;
    }
    isLet() {
        return this.flags === common_1.ts.NodeFlags.Let;
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], VariableDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], VariableDeclaration.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], VariableDeclaration.prototype, "decorators", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], VariableDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], VariableDeclaration.prototype, "exclamationToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], VariableDeclaration.prototype, "type", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], VariableDeclaration.prototype, "initializer", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], VariableDeclaration.prototype, "modifiers", void 0);
VariableDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.VariableDeclaration
    })
], VariableDeclaration);
exports.VariableDeclaration = VariableDeclaration;
let DoStatement = class DoStatement extends Node {
    visit(visitor, context) {
        return visitor.visitDoStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], DoStatement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], DoStatement.prototype, "expression", void 0);
DoStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.DoStatement
    })
], DoStatement);
exports.DoStatement = DoStatement;
let WhileStatement = class WhileStatement extends Node {
    visit(visitor, context) {
        return visitor.visitWhileStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], WhileStatement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], WhileStatement.prototype, "expression", void 0);
WhileStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.WhileStatement
    })
], WhileStatement);
exports.WhileStatement = WhileStatement;
let ForStatement = class ForStatement extends Node {
    visit(visitor, context) {
        return visitor.visitForStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ForStatement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ForStatement.prototype, "initializer", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ForStatement.prototype, "condition", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ForStatement.prototype, "incrementor", void 0);
ForStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ForStatement
    })
], ForStatement);
exports.ForStatement = ForStatement;
let ForInStatement = class ForInStatement extends Node {
    visit(visitor, context) {
        return visitor.visitForInStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ForInStatement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ForInStatement.prototype, "initializer", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ForInStatement.prototype, "expression", void 0);
ForInStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ForInStatement
    })
], ForInStatement);
exports.ForInStatement = ForInStatement;
let ForOfStatement = class ForOfStatement extends Node {
    visit(visitor, context) {
        return visitor.visitForOfStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ForOfStatement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], ForOfStatement.prototype, "awaitModifier", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ForOfStatement.prototype, "initializer", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ForOfStatement.prototype, "expression", void 0);
ForOfStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ForOfStatement
    })
], ForOfStatement);
exports.ForOfStatement = ForOfStatement;
let BreakStatement = class BreakStatement extends Node {
    visit(visitor, context) {
        return visitor.visitBreakStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], BreakStatement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Identifier)
], BreakStatement.prototype, "label", void 0);
BreakStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.BreakStatement
    })
], BreakStatement);
exports.BreakStatement = BreakStatement;
let ContinueStatement = class ContinueStatement extends Node {
    visit(visitor, context) {
        return visitor.visitContinueStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ContinueStatement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Identifier)
], ContinueStatement.prototype, "label", void 0);
ContinueStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ContinueStatement
    })
], ContinueStatement);
exports.ContinueStatement = ContinueStatement;
let ReturnStatement = class ReturnStatement extends Node {
    visit(visitor, context) {
        return visitor.visitReturnStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ReturnStatement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ReturnStatement.prototype, "expression", void 0);
ReturnStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ReturnStatement
    })
], ReturnStatement);
exports.ReturnStatement = ReturnStatement;
let WithStatement = class WithStatement extends Node {
    visit(visitor, context) {
        return visitor.visitWithStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], WithStatement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], WithStatement.prototype, "expression", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], WithStatement.prototype, "statement", void 0);
WithStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.WithStatement
    })
], WithStatement);
exports.WithStatement = WithStatement;
let LabeledStatement = class LabeledStatement extends Node {
    visit(visitor, context) {
        return visitor.visitLabeledStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], LabeledStatement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Identifier)
], LabeledStatement.prototype, "label", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], LabeledStatement.prototype, "statement", void 0);
LabeledStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.LabeledStatement
    })
], LabeledStatement);
exports.LabeledStatement = LabeledStatement;
let CaseBlock = class CaseBlock extends Node {
    visit(visitor, context) {
        return visitor.visitCaseBlock(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], CaseBlock.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], CaseBlock.prototype, "clauses", void 0);
CaseBlock = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.CaseBlock
    })
], CaseBlock);
exports.CaseBlock = CaseBlock;
let SwitchStatement = class SwitchStatement extends Node {
    visit(visitor, context) {
        return visitor.visitSwitchStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], SwitchStatement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], SwitchStatement.prototype, "expression", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", CaseBlock)
], SwitchStatement.prototype, "caseBlock", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Boolean)
], SwitchStatement.prototype, "possiblyExhaustive", void 0);
SwitchStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.SwitchStatement
    })
], SwitchStatement);
exports.SwitchStatement = SwitchStatement;
let ThrowStatement = class ThrowStatement extends Node {
    visit(visitor, context) {
        return visitor.visitThrowStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ThrowStatement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ThrowStatement.prototype, "expression", void 0);
ThrowStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ThrowStatement
    })
], ThrowStatement);
exports.ThrowStatement = ThrowStatement;
let CatchClause = class CatchClause extends Node {
    visit(visitor, context) {
        return visitor.visitCatchClause(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], CatchClause.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", VariableDeclaration)
], CatchClause.prototype, "variableDeclaration", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Block)
], CatchClause.prototype, "block", void 0);
CatchClause = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.CatchClause
    })
], CatchClause);
exports.CatchClause = CatchClause;
let TryStatement = class TryStatement extends Node {
    visit(visitor, context) {
        return visitor.visitTryStatement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TryStatement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Block)
], TryStatement.prototype, "tryBlock", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", CatchClause)
], TryStatement.prototype, "catchClause", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Block)
], TryStatement.prototype, "finallyBlock", void 0);
TryStatement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TryStatement
    })
], TryStatement);
exports.TryStatement = TryStatement;
let ImportClause = class ImportClause extends Node {
    visit(visitor, context) {
        return visitor.visitImportClause(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ImportClause.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Identifier)
], ImportClause.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ImportClause.prototype, "namedBindings", void 0);
ImportClause = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ImportClause
    })
], ImportClause);
exports.ImportClause = ImportClause;
let ImportDeclaration = class ImportDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitImportDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ImportDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ImportDeclaration.prototype, "moduleSpecifier", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", ImportClause)
], ImportDeclaration.prototype, "importClause", void 0);
ImportDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ImportDeclaration
    })
], ImportDeclaration);
exports.ImportDeclaration = ImportDeclaration;
let ModuleBlock = class ModuleBlock extends Node {
    visit(visitor, context) {
        return visitor.visitModuleBlock(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ModuleBlock.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], ModuleBlock.prototype, "statements", void 0);
ModuleBlock = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ModuleBlock
    })
], ModuleBlock);
exports.ModuleBlock = ModuleBlock;
let JSDocNamespaceDeclaration = class JSDocNamespaceDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocNamespaceDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Identifier)
], JSDocNamespaceDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], JSDocNamespaceDeclaration.prototype, "body", void 0);
JSDocNamespaceDeclaration = tslib_1.__decorate([
    core_1.Plain()
], JSDocNamespaceDeclaration);
exports.JSDocNamespaceDeclaration = JSDocNamespaceDeclaration;
let JsxSpreadAttribute = class JsxSpreadAttribute extends Node {
    visit(visitor, context) {
        return visitor.visitJsxSpreadAttribute(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JsxSpreadAttribute.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], JsxSpreadAttribute.prototype, "expression", void 0);
JsxSpreadAttribute = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JsxSpreadAttribute
    })
], JsxSpreadAttribute);
exports.JsxSpreadAttribute = JsxSpreadAttribute;
let StringLiteral = class StringLiteral extends Node {
    visit(visitor, context) {
        return visitor.visitStringLiteral(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], StringLiteral.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], StringLiteral.prototype, "text", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], StringLiteral.prototype, "isUnterminated", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], StringLiteral.prototype, "hasExtendedUnicodeEscape", void 0);
StringLiteral = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.StringLiteral
    })
], StringLiteral);
exports.StringLiteral = StringLiteral;
let JsxAttribute = class JsxAttribute extends Node {
    visit(visitor, context) {
        return visitor.visitJsxAttribute(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JsxAttribute.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Identifier)
], JsxAttribute.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], JsxAttribute.prototype, "initializer", void 0);
JsxAttribute = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JsxAttribute
    })
], JsxAttribute);
exports.JsxAttribute = JsxAttribute;
let ObjectLiteralExpression = class ObjectLiteralExpression extends Node {
    visit(visitor, context) {
        return visitor.visitObjectLiteralExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ObjectLiteralExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], ObjectLiteralExpression.prototype, "properties", void 0);
ObjectLiteralExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ObjectLiteralExpression
    })
], ObjectLiteralExpression);
exports.ObjectLiteralExpression = ObjectLiteralExpression;
let JsxAttributes = class JsxAttributes extends Node {
    visit(visitor, context) {
        return visitor.visitJsxAttributes(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JsxAttributes.prototype, "kind", void 0);
JsxAttributes = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JsxAttributes
    })
], JsxAttributes);
exports.JsxAttributes = JsxAttributes;
let JsxOpeningFragment = class JsxOpeningFragment extends Node {
    visit(visitor, context) {
        return visitor.visitJsxOpeningFragment(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JsxOpeningFragment.prototype, "kind", void 0);
JsxOpeningFragment = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JsxOpeningFragment
    })
], JsxOpeningFragment);
exports.JsxOpeningFragment = JsxOpeningFragment;
let JsxClosingFragment = class JsxClosingFragment extends Node {
    visit(visitor, context) {
        return visitor.visitJsxClosingFragment(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JsxClosingFragment.prototype, "kind", void 0);
JsxClosingFragment = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JsxClosingFragment
    })
], JsxClosingFragment);
exports.JsxClosingFragment = JsxClosingFragment;
let JsxFragment = class JsxFragment extends Node {
    visit(visitor, context) {
        return visitor.visitJsxFragment(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JsxFragment.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", JsxOpeningFragment)
], JsxFragment.prototype, "openingFragment", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], JsxFragment.prototype, "children", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", JsxClosingFragment)
], JsxFragment.prototype, "closingFragment", void 0);
JsxFragment = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JsxFragment
    })
], JsxFragment);
exports.JsxFragment = JsxFragment;
let JsxSelfClosingElement = class JsxSelfClosingElement extends Node {
    visit(visitor, context) {
        return visitor.visitJsxSelfClosingElement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JsxSelfClosingElement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], JsxSelfClosingElement.prototype, "tagName", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], JsxSelfClosingElement.prototype, "typeArguments", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", JsxAttributes)
], JsxSelfClosingElement.prototype, "attributes", void 0);
JsxSelfClosingElement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JsxSelfClosingElement
    })
], JsxSelfClosingElement);
exports.JsxSelfClosingElement = JsxSelfClosingElement;
let JsxOpeningElement = class JsxOpeningElement extends Node {
    visit(visitor, context) {
        return visitor.visitJsxOpeningElement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JsxOpeningElement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], JsxOpeningElement.prototype, "tagName", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], JsxOpeningElement.prototype, "typeArguments", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", JsxAttributes)
], JsxOpeningElement.prototype, "attributes", void 0);
JsxOpeningElement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JsxOpeningElement
    })
], JsxOpeningElement);
exports.JsxOpeningElement = JsxOpeningElement;
let JsxClosingElement = class JsxClosingElement extends Node {
    visit(visitor, context) {
        return visitor.visitJsxClosingElement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JsxClosingElement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], JsxClosingElement.prototype, "tagName", void 0);
JsxClosingElement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JsxClosingElement
    })
], JsxClosingElement);
exports.JsxClosingElement = JsxClosingElement;
let JsxElement = class JsxElement extends Node {
    visit(visitor, context) {
        return visitor.visitJsxElement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JsxElement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", JsxOpeningElement)
], JsxElement.prototype, "openingElement", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], JsxElement.prototype, "children", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", JsxClosingElement)
], JsxElement.prototype, "closingElement", void 0);
JsxElement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JsxElement
    })
], JsxElement);
exports.JsxElement = JsxElement;
let MetaProperty = class MetaProperty extends Node {
    visit(visitor, context) {
        return visitor.visitMetaProperty(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], MetaProperty.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], MetaProperty.prototype, "keywordToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], MetaProperty.prototype, "name", void 0);
MetaProperty = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.MetaProperty
    })
], MetaProperty);
exports.MetaProperty = MetaProperty;
let ArrayLiteralExpression = class ArrayLiteralExpression extends Node {
    visit(visitor, context) {
        return visitor.visitArrayLiteralExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ArrayLiteralExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ArrayLiteralExpression.prototype, "elements", void 0);
ArrayLiteralExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ArrayLiteralExpression
    })
], ArrayLiteralExpression);
exports.ArrayLiteralExpression = ArrayLiteralExpression;
let ParenthesizedExpression = class ParenthesizedExpression extends Node {
    visit(visitor, context) {
        return visitor.visitParenthesizedExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ParenthesizedExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ParenthesizedExpression.prototype, "expression", void 0);
ParenthesizedExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ParenthesizedExpression
    })
], ParenthesizedExpression);
exports.ParenthesizedExpression = ParenthesizedExpression;
let TemplateHead = class TemplateHead extends Node {
    visit(visitor, context) {
        return visitor.visitTemplateHead(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TemplateHead.prototype, "kind", void 0);
TemplateHead = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TemplateHead
    })
], TemplateHead);
exports.TemplateHead = TemplateHead;
let TemplateExpression = class TemplateExpression extends Node {
    visit(visitor, context) {
        return visitor.visitTemplateExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TemplateExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", TemplateHead)
], TemplateExpression.prototype, "head", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], TemplateExpression.prototype, "templateSpans", void 0);
TemplateExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TemplateExpression
    })
], TemplateExpression);
exports.TemplateExpression = TemplateExpression;
let TemplateSpan = class TemplateSpan extends Node {
    visit(visitor, context) {
        return visitor.visitTemplateSpan(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TemplateSpan.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], TemplateSpan.prototype, "expression", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], TemplateSpan.prototype, "literal", void 0);
TemplateSpan = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TemplateSpan
    })
], TemplateSpan);
exports.TemplateSpan = TemplateSpan;
let TemplateMiddle = class TemplateMiddle extends Node {
    visit(visitor, context) {
        return visitor.visitTemplateMiddle(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], TemplateMiddle.prototype, "rawText", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TemplateMiddle.prototype, "kind", void 0);
TemplateMiddle = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TemplateMiddle
    })
], TemplateMiddle);
exports.TemplateMiddle = TemplateMiddle;
let FunctionExpression = class FunctionExpression extends Node {
    visit(visitor, context) {
        return visitor.visitFunctionExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], FunctionExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Identifier)
], FunctionExpression.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], FunctionExpression.prototype, "body", void 0);
FunctionExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.FunctionExpression
    })
], FunctionExpression);
exports.FunctionExpression = FunctionExpression;
let TemplateTail = class TemplateTail extends Node {
    visit(visitor, context) {
        return visitor.visitTemplateTail(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TemplateTail.prototype, "kind", void 0);
TemplateTail = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TemplateTail
    })
], TemplateTail);
exports.TemplateTail = TemplateTail;
let ThisExpression = class ThisExpression extends Node {
    visit(visitor, context) {
        return visitor.visitThisExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ThisExpression.prototype, "kind", void 0);
ThisExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ThisKeyword
    })
], ThisExpression);
exports.ThisExpression = ThisExpression;
let SuperExpression = class SuperExpression extends Node {
    visit(visitor, context) {
        return visitor.visitSuperExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], SuperExpression.prototype, "kind", void 0);
SuperExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.SuperKeyword
    })
], SuperExpression);
exports.SuperExpression = SuperExpression;
let ImportExpression = class ImportExpression extends Node {
    visit(visitor, context) {
        return visitor.visitImportExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ImportExpression.prototype, "kind", void 0);
ImportExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ImportKeyword
    })
], ImportExpression);
exports.ImportExpression = ImportExpression;
let NullLiteral = class NullLiteral extends Node {
    visit(visitor, context) {
        return visitor.visitNullLiteral(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], NullLiteral.prototype, "kind", void 0);
NullLiteral = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.NullKeyword
    })
], NullLiteral);
exports.NullLiteral = NullLiteral;
let BooleanLiteral = class BooleanLiteral extends Node {
    isTrue() {
        return this.kind === common_1.ts.SyntaxKind.TrueKeyword;
    }
    visit(visitor, context) {
        return visitor.visitBooleanLiteral(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], BooleanLiteral.prototype, "kind", void 0);
BooleanLiteral = tslib_1.__decorate([
    core_1.Plain({
        desc: [common_1.ts.SyntaxKind.TrueKeyword, common_1.ts.SyntaxKind.FalseKeyword]
    })
], BooleanLiteral);
exports.BooleanLiteral = BooleanLiteral;
let PartiallyEmittedExpression = class PartiallyEmittedExpression extends Node {
    visit(visitor, context) {
        return visitor.visitPartiallyEmittedExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], PartiallyEmittedExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], PartiallyEmittedExpression.prototype, "expression", void 0);
PartiallyEmittedExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.PartiallyEmittedExpression
    })
], PartiallyEmittedExpression);
exports.PartiallyEmittedExpression = PartiallyEmittedExpression;
let NonNullExpression = class NonNullExpression extends Node {
    visit(visitor, context) {
        return visitor.visitNonNullExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], NonNullExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], NonNullExpression.prototype, "expression", void 0);
NonNullExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.NonNullExpression
    })
], NonNullExpression);
exports.NonNullExpression = NonNullExpression;
let PostfixUnaryExpression = class PostfixUnaryExpression extends Node {
    visit(visitor, context) {
        return visitor.visitPostfixUnaryExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], PostfixUnaryExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], PostfixUnaryExpression.prototype, "operand", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Number)
], PostfixUnaryExpression.prototype, "operator", void 0);
PostfixUnaryExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.PostfixUnaryExpression
    })
], PostfixUnaryExpression);
exports.PostfixUnaryExpression = PostfixUnaryExpression;
let PrefixUnaryExpression = class PrefixUnaryExpression extends Node {
    visit(visitor, context) {
        return visitor.visitPrefixUnaryExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], PrefixUnaryExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], PrefixUnaryExpression.prototype, "operator", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], PrefixUnaryExpression.prototype, "operand", void 0);
PrefixUnaryExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.PrefixUnaryExpression
    })
], PrefixUnaryExpression);
exports.PrefixUnaryExpression = PrefixUnaryExpression;
let DeleteExpression = class DeleteExpression extends Node {
    visit(visitor, context) {
        return visitor.visitDeleteExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], DeleteExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], DeleteExpression.prototype, "expression", void 0);
DeleteExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.DeleteExpression
    })
], DeleteExpression);
exports.DeleteExpression = DeleteExpression;
let TypeOfExpression = class TypeOfExpression extends Node {
    visit(visitor, context) {
        return visitor.visitTypeOfExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypeOfExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], TypeOfExpression.prototype, "expression", void 0);
TypeOfExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TypeOfExpression
    })
], TypeOfExpression);
exports.TypeOfExpression = TypeOfExpression;
let VoidExpression = class VoidExpression extends Node {
    visit(visitor, context) {
        return visitor.visitVoidExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], VoidExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], VoidExpression.prototype, "expression", void 0);
VoidExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.VoidExpression
    })
], VoidExpression);
exports.VoidExpression = VoidExpression;
let AwaitExpression = class AwaitExpression extends Node {
    visit(visitor, context) {
        return visitor.visitAwaitExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], AwaitExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], AwaitExpression.prototype, "expression", void 0);
AwaitExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.AwaitExpression
    })
], AwaitExpression);
exports.AwaitExpression = AwaitExpression;
let TypeAssertion = class TypeAssertion extends Node {
    visit(visitor, context) {
        return visitor.visitTypeAssertion(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypeAssertion.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], TypeAssertion.prototype, "type", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], TypeAssertion.prototype, "expression", void 0);
TypeAssertion = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TypeAssertionExpression
    })
], TypeAssertion);
exports.TypeAssertion = TypeAssertion;
let CommaListExpression = class CommaListExpression extends Node {
    visit(visitor, context) {
        return visitor.visitCommaListExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], CommaListExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], CommaListExpression.prototype, "elements", void 0);
CommaListExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.CommaListExpression
    })
], CommaListExpression);
exports.CommaListExpression = CommaListExpression;
let JsxExpression = class JsxExpression extends Node {
    visit(visitor, context) {
        return visitor.visitJsxExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JsxExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], JsxExpression.prototype, "dotDotDotToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], JsxExpression.prototype, "expression", void 0);
JsxExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JsxExpression
    })
], JsxExpression);
exports.JsxExpression = JsxExpression;
let AsExpression = class AsExpression extends Node {
    visit(visitor, context) {
        return visitor.visitAsExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], AsExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], AsExpression.prototype, "expression", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], AsExpression.prototype, "type", void 0);
AsExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.AsExpression
    })
], AsExpression);
exports.AsExpression = AsExpression;
let CallExpression = class CallExpression extends Node {
    visit(visitor, context) {
        return visitor.visitCallExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], CallExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], CallExpression.prototype, "expression", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], CallExpression.prototype, "questionDotToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], CallExpression.prototype, "typeArguments", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], CallExpression.prototype, "arguments", void 0);
CallExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.CallExpression
    })
], CallExpression);
exports.CallExpression = CallExpression;
let NewExpression = class NewExpression extends Node {
    visit(visitor, context) {
        return visitor.visitNewExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], NewExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], NewExpression.prototype, "expression", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], NewExpression.prototype, "typeArguments", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], NewExpression.prototype, "arguments", void 0);
NewExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.NewExpression
    })
], NewExpression);
exports.NewExpression = NewExpression;
let SpreadElement = class SpreadElement extends Node {
    visit(visitor, context) {
        return visitor.visitSpreadElement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], SpreadElement.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], SpreadElement.prototype, "expression", void 0);
SpreadElement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.SpreadElement
    })
], SpreadElement);
exports.SpreadElement = SpreadElement;
let ConditionalExpression = class ConditionalExpression extends Node {
    visit(visitor, context) {
        return visitor.visitConditionalExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ConditionalExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ConditionalExpression.prototype, "condition", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], ConditionalExpression.prototype, "questionToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ConditionalExpression.prototype, "whenTrue", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], ConditionalExpression.prototype, "colonToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ConditionalExpression.prototype, "whenFalse", void 0);
ConditionalExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ConditionalExpression
    })
], ConditionalExpression);
exports.ConditionalExpression = ConditionalExpression;
let ElementAccessExpression = class ElementAccessExpression extends Node {
    visit(visitor, context) {
        return visitor.visitElementAccessExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ElementAccessExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ElementAccessExpression.prototype, "expression", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], ElementAccessExpression.prototype, "questionDotToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ElementAccessExpression.prototype, "argumentExpression", void 0);
ElementAccessExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ElementAccessExpression
    })
], ElementAccessExpression);
exports.ElementAccessExpression = ElementAccessExpression;
let OmittedExpression = class OmittedExpression extends Node {
    visit(visitor, context) {
        return visitor.visitOmittedExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], OmittedExpression.prototype, "kind", void 0);
OmittedExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.OmittedExpression
    })
], OmittedExpression);
exports.OmittedExpression = OmittedExpression;
let YieldExpression = class YieldExpression extends Node {
    visit(visitor, context) {
        return visitor.visitYieldExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], YieldExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], YieldExpression.prototype, "asteriskToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], YieldExpression.prototype, "expression", void 0);
YieldExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.YieldExpression
    })
], YieldExpression);
exports.YieldExpression = YieldExpression;
let SyntheticExpression = class SyntheticExpression extends Node {
    visit(visitor, context) {
        return visitor.visitSyntheticExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], SyntheticExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], SyntheticExpression.prototype, "isSpread", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], SyntheticExpression.prototype, "type", void 0);
SyntheticExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.SyntheticExpression
    })
], SyntheticExpression);
exports.SyntheticExpression = SyntheticExpression;
let PropertyAccessExpression = class PropertyAccessExpression extends Node {
    visit(visitor, context) {
        return visitor.visitPropertyAccessExpression(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], PropertyAccessExpression.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], PropertyAccessExpression.prototype, "expression", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], PropertyAccessExpression.prototype, "questionDotToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Identifier)
], PropertyAccessExpression.prototype, "name", void 0);
PropertyAccessExpression = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.PropertyAccessExpression
    })
], PropertyAccessExpression);
exports.PropertyAccessExpression = PropertyAccessExpression;
let BigIntLiteral = class BigIntLiteral extends Node {
    visit(visitor, context) {
        return visitor.visitBigIntLiteral(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], BigIntLiteral.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], BigIntLiteral.prototype, "text", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], BigIntLiteral.prototype, "isUnterminated", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], BigIntLiteral.prototype, "hasExtendedUnicodeEscape", void 0);
BigIntLiteral = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.BigIntLiteral
    })
], BigIntLiteral);
exports.BigIntLiteral = BigIntLiteral;
let NumericLiteral = class NumericLiteral extends Node {
    visit(visitor, context) {
        return visitor.visitNumericLiteral(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], NumericLiteral.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], NumericLiteral.prototype, "text", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], NumericLiteral.prototype, "isUnterminated", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], NumericLiteral.prototype, "hasExtendedUnicodeEscape", void 0);
NumericLiteral = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.NumericLiteral
    })
], NumericLiteral);
exports.NumericLiteral = NumericLiteral;
let ClassDeclaration = class ClassDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitClassDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ClassDeclaration.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], ClassDeclaration.prototype, "decorators", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], ClassDeclaration.prototype, "modifiers", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ClassDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Identifier)
], ClassDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], ClassDeclaration.prototype, "typeParameters", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], ClassDeclaration.prototype, "heritageClauses", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Array)
], ClassDeclaration.prototype, "members", void 0);
ClassDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ClassDeclaration
    })
], ClassDeclaration);
exports.ClassDeclaration = ClassDeclaration;
let PropertyDeclaration = class PropertyDeclaration extends Node {
    visit(visitor, context) {
        return visitor.visitPropertyDeclaration(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], PropertyDeclaration.prototype, "modifiers", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], PropertyDeclaration.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], PropertyDeclaration.prototype, "decorators", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], PropertyDeclaration.prototype, "jsDoc", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], PropertyDeclaration.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], PropertyDeclaration.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], PropertyDeclaration.prototype, "questionToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], PropertyDeclaration.prototype, "exclamationToken", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], PropertyDeclaration.prototype, "type", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], PropertyDeclaration.prototype, "initializer", void 0);
PropertyDeclaration = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.PropertyDeclaration
    })
], PropertyDeclaration);
exports.PropertyDeclaration = PropertyDeclaration;
let SemicolonClassElement = class SemicolonClassElement extends Node {
    visit(visitor, context) {
        return visitor.visitSemicolonClassElement(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], SemicolonClassElement.prototype, "kind", void 0);
SemicolonClassElement = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.SemicolonClassElement
    })
], SemicolonClassElement);
exports.SemicolonClassElement = SemicolonClassElement;
let HeritageClause = class HeritageClause extends Node {
    getToken() {
        return this.token === common_1.ts.SyntaxKind.ExtendsKeyword ? 'extends' : 'implements';
    }
    visit(visitor, context) {
        return visitor.visitHeritageClause(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], HeritageClause.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], HeritageClause.prototype, "token", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], HeritageClause.prototype, "types", void 0);
HeritageClause = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.HeritageClause
    })
], HeritageClause);
exports.HeritageClause = HeritageClause;
let ExportAssignment = class ExportAssignment extends Node {
    visit(visitor, context) {
        return visitor.visitExportAssignment(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ExportAssignment.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], ExportAssignment.prototype, "isExportEquals", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ExportAssignment.prototype, "expression", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], ExportAssignment.prototype, "name", void 0);
ExportAssignment = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ExportAssignment
    })
], ExportAssignment);
exports.ExportAssignment = ExportAssignment;
let ConditionalTypeNode = class ConditionalTypeNode extends Node {
    visit(visitor, context) {
        return visitor.visitConditionalTypeNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ConditionalTypeNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ConditionalTypeNode.prototype, "checkType", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ConditionalTypeNode.prototype, "extendsType", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ConditionalTypeNode.prototype, "trueType", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ConditionalTypeNode.prototype, "falseType", void 0);
ConditionalTypeNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.ConditionalType
    })
], ConditionalTypeNode);
exports.ConditionalTypeNode = ConditionalTypeNode;
let InferTypeNode = class InferTypeNode extends Node {
    visit(visitor, context) {
        return visitor.visitInferTypeNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], InferTypeNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", TypeParameterDeclaration)
], InferTypeNode.prototype, "typeParameter", void 0);
InferTypeNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.InferType
    })
], InferTypeNode);
exports.InferTypeNode = InferTypeNode;
let UnionTypeNode = class UnionTypeNode extends Node {
    visit(visitor, context) {
        return visitor.visitUnionTypeNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], UnionTypeNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], UnionTypeNode.prototype, "types", void 0);
UnionTypeNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.UnionType
    })
], UnionTypeNode);
exports.UnionTypeNode = UnionTypeNode;
let IntersectionTypeNode = class IntersectionTypeNode extends Node {
    visit(visitor, context) {
        return visitor.visitIntersectionTypeNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], IntersectionTypeNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], IntersectionTypeNode.prototype, "types", void 0);
IntersectionTypeNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.IntersectionType
    })
], IntersectionTypeNode);
exports.IntersectionTypeNode = IntersectionTypeNode;
let IndexedAccessTypeNode = class IndexedAccessTypeNode extends Node {
    visit(visitor, context) {
        return visitor.visitIndexedAccessTypeNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], IndexedAccessTypeNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], IndexedAccessTypeNode.prototype, "objectType", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], IndexedAccessTypeNode.prototype, "indexType", void 0);
IndexedAccessTypeNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.IndexedAccessType
    })
], IndexedAccessTypeNode);
exports.IndexedAccessTypeNode = IndexedAccessTypeNode;
let RegularExpressionLiteral = class RegularExpressionLiteral extends Node {
    visit(visitor, context) {
        return visitor.visitRegularExpressionLiteral(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], RegularExpressionLiteral.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], RegularExpressionLiteral.prototype, "text", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], RegularExpressionLiteral.prototype, "isUnterminated", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Boolean)
], RegularExpressionLiteral.prototype, "hasExtendedUnicodeEscape", void 0);
RegularExpressionLiteral = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.RegularExpressionLiteral
    })
], RegularExpressionLiteral);
exports.RegularExpressionLiteral = RegularExpressionLiteral;
let LiteralTypeNode = class LiteralTypeNode extends Node {
    visit(visitor, context) {
        return visitor.visitLiteralTypeNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], LiteralTypeNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], LiteralTypeNode.prototype, "literal", void 0);
LiteralTypeNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.LiteralType
    })
], LiteralTypeNode);
exports.LiteralTypeNode = LiteralTypeNode;
let TypeQueryNode = class TypeQueryNode extends Node {
    visit(visitor, context) {
        return visitor.visitTypeQueryNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypeQueryNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], TypeQueryNode.prototype, "exprName", void 0);
TypeQueryNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TypeQuery
    })
], TypeQueryNode);
exports.TypeQueryNode = TypeQueryNode;
let JSDocAllType = class JSDocAllType extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocAllType(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocAllType.prototype, "kind", void 0);
JSDocAllType = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocAllType
    })
], JSDocAllType);
exports.JSDocAllType = JSDocAllType;
let JSDocUnknownType = class JSDocUnknownType extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocUnknownType(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocUnknownType.prototype, "kind", void 0);
JSDocUnknownType = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocUnknownType
    })
], JSDocUnknownType);
exports.JSDocUnknownType = JSDocUnknownType;
let JSDocNamepathType = class JSDocNamepathType extends Node {
    visit(visitor, context) {
        return visitor.visitJSDocNamepathType(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], JSDocNamepathType.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], JSDocNamepathType.prototype, "type", void 0);
JSDocNamepathType = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.JSDocNamepathType
    })
], JSDocNamepathType);
exports.JSDocNamepathType = JSDocNamepathType;
let ArrayTypeNode = class ArrayTypeNode extends Node {
    visit(visitor, context) {
        return visitor.visitArrayTypeNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ArrayTypeNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], ArrayTypeNode.prototype, "elementType", void 0);
ArrayTypeNode = tslib_1.__decorate([
    core_1.Plain({
        desc: { kind: common_1.ts.SyntaxKind.ArrayType }
    })
], ArrayTypeNode);
exports.ArrayTypeNode = ArrayTypeNode;
let TupleTypeNode = class TupleTypeNode extends Node {
    visit(visitor, context) {
        return visitor.visitTupleTypeNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TupleTypeNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], TupleTypeNode.prototype, "elementTypes", void 0);
TupleTypeNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.TupleType
    })
], TupleTypeNode);
exports.TupleTypeNode = TupleTypeNode;
let OptionalTypeNode = class OptionalTypeNode extends Node {
    visit(visitor, context) {
        return visitor.visitOptionalTypeNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], OptionalTypeNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], OptionalTypeNode.prototype, "type", void 0);
OptionalTypeNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.OptionalType
    })
], OptionalTypeNode);
exports.OptionalTypeNode = OptionalTypeNode;
let RestTypeNode = class RestTypeNode extends Node {
    visit(visitor, context) {
        return visitor.visitRestTypeNode(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], RestTypeNode.prototype, "kind", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], RestTypeNode.prototype, "type", void 0);
RestTypeNode = tslib_1.__decorate([
    core_1.Plain({
        desc: common_1.ts.SyntaxKind.RestType
    })
], RestTypeNode);
exports.RestTypeNode = RestTypeNode;
let FunctionScopedVariableSymbol = class FunctionScopedVariableSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitFunctionScopedVariableSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], FunctionScopedVariableSymbol.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], FunctionScopedVariableSymbol.prototype, "intrinsicName", void 0);
FunctionScopedVariableSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.FunctionScopedVariable }
    })
], FunctionScopedVariableSymbol);
exports.FunctionScopedVariableSymbol = FunctionScopedVariableSymbol;
let BlockScopedVariableSymbol = class BlockScopedVariableSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitBlockScopedVariableSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], BlockScopedVariableSymbol.prototype, "flags", void 0);
BlockScopedVariableSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.BlockScopedVariable }
    })
], BlockScopedVariableSymbol);
exports.BlockScopedVariableSymbol = BlockScopedVariableSymbol;
let PropertySymbol = class PropertySymbol extends Node {
    visit(visitor, context) {
        return visitor.visitPropertySymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], PropertySymbol.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], PropertySymbol.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], PropertySymbol.prototype, "escapedName", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", PropertySignature)
], PropertySymbol.prototype, "valueDeclaration", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], PropertySymbol.prototype, "intrinsicName", void 0);
PropertySymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Property }
    })
], PropertySymbol);
exports.PropertySymbol = PropertySymbol;
class PropertyType {
}
exports.PropertyType = PropertyType;
let EnumMemberSymbol = class EnumMemberSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitEnumMemberSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], EnumMemberSymbol.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], EnumMemberSymbol.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], EnumMemberSymbol.prototype, "escapedName", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Node)
], EnumMemberSymbol.prototype, "valueDeclaration", void 0);
EnumMemberSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.EnumMember }
    })
], EnumMemberSymbol);
exports.EnumMemberSymbol = EnumMemberSymbol;
let FunctionSymbol = class FunctionSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitFunctionSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], FunctionSymbol.prototype, "declarations", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], FunctionSymbol.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], FunctionSymbol.prototype, "id", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], FunctionSymbol.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", FunctionDeclaration)
], FunctionSymbol.prototype, "valueDeclaration", void 0);
FunctionSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Function }
    })
], FunctionSymbol);
exports.FunctionSymbol = FunctionSymbol;
let ClassSymbol = class ClassSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitClassSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], ClassSymbol.prototype, "declarations", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ClassSymbol.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], ClassSymbol.prototype, "escapedName", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], ClassSymbol.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ClassSymbol.prototype, "id", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Map)
], ClassSymbol.prototype, "members", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", ClassDeclaration)
], ClassSymbol.prototype, "valueDeclaration", void 0);
ClassSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Class }
    })
], ClassSymbol);
exports.ClassSymbol = ClassSymbol;
let InterfaceSymbol = class InterfaceSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitInterfaceSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], InterfaceSymbol.prototype, "declarations", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], InterfaceSymbol.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], InterfaceSymbol.prototype, "id", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], InterfaceSymbol.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], InterfaceSymbol.prototype, "escapedName", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Map)
], InterfaceSymbol.prototype, "members", void 0);
InterfaceSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Interface }
    })
], InterfaceSymbol);
exports.InterfaceSymbol = InterfaceSymbol;
let ConstEnumSymbol = class ConstEnumSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitConstEnumSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ConstEnumSymbol.prototype, "flags", void 0);
ConstEnumSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.ConstEnum }
    })
], ConstEnumSymbol);
exports.ConstEnumSymbol = ConstEnumSymbol;
let RegularEnumSymbol = class RegularEnumSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitRegularEnumSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], RegularEnumSymbol.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], RegularEnumSymbol.prototype, "escapedName", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], RegularEnumSymbol.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", EnumDeclaration)
], RegularEnumSymbol.prototype, "valueDeclaration", void 0);
RegularEnumSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.RegularEnum }
    })
], RegularEnumSymbol);
exports.RegularEnumSymbol = RegularEnumSymbol;
let ValueModuleSymbol = class ValueModuleSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitValueModuleSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ValueModuleSymbol.prototype, "flags", void 0);
ValueModuleSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.ValueModule }
    })
], ValueModuleSymbol);
exports.ValueModuleSymbol = ValueModuleSymbol;
let NamespaceModuleSymbol = class NamespaceModuleSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitNamespaceModuleSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], NamespaceModuleSymbol.prototype, "flags", void 0);
NamespaceModuleSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.NamespaceModule }
    })
], NamespaceModuleSymbol);
exports.NamespaceModuleSymbol = NamespaceModuleSymbol;
let TypeLiteralSymbol = class TypeLiteralSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitTypeLiteralSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypeLiteralSymbol.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Map)
], TypeLiteralSymbol.prototype, "members", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], TypeLiteralSymbol.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], TypeLiteralSymbol.prototype, "escapedName", void 0);
TypeLiteralSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.TypeLiteral }
    })
], TypeLiteralSymbol);
exports.TypeLiteralSymbol = TypeLiteralSymbol;
let ObjectLiteralSymbol = class ObjectLiteralSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitObjectLiteralSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ObjectLiteralSymbol.prototype, "flags", void 0);
ObjectLiteralSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.ObjectLiteral }
    })
], ObjectLiteralSymbol);
exports.ObjectLiteralSymbol = ObjectLiteralSymbol;
let MethodSymbol = class MethodSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitMethodSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], MethodSymbol.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], MethodSymbol.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], MethodSymbol.prototype, "escapedName", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", MethodSignature)
], MethodSymbol.prototype, "valueDeclaration", void 0);
MethodSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Method }
    })
], MethodSymbol);
exports.MethodSymbol = MethodSymbol;
let ConstructorSymbol = class ConstructorSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitConstructorSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ConstructorSymbol.prototype, "flags", void 0);
ConstructorSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Constructor }
    })
], ConstructorSymbol);
exports.ConstructorSymbol = ConstructorSymbol;
let GetAccessorSymbol = class GetAccessorSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitGetAccessorSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], GetAccessorSymbol.prototype, "flags", void 0);
GetAccessorSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.GetAccessor }
    })
], GetAccessorSymbol);
exports.GetAccessorSymbol = GetAccessorSymbol;
let SetAccessorSymbol = class SetAccessorSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitSetAccessorSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], SetAccessorSymbol.prototype, "flags", void 0);
SetAccessorSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.SetAccessor }
    })
], SetAccessorSymbol);
exports.SetAccessorSymbol = SetAccessorSymbol;
let SignatureSymbol = class SignatureSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitSignatureSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], SignatureSymbol.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], SignatureSymbol.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], SignatureSymbol.prototype, "escapedName", void 0);
SignatureSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Signature }
    })
], SignatureSymbol);
exports.SignatureSymbol = SignatureSymbol;
let TypeParameterSymbol = class TypeParameterSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitTypeParameterSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypeParameterSymbol.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], TypeParameterSymbol.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], TypeParameterSymbol.prototype, "escapedName", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], TypeParameterSymbol.prototype, "declarations", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypeParameterSymbol.prototype, "id", void 0);
TypeParameterSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.TypeParameter }
    })
], TypeParameterSymbol);
exports.TypeParameterSymbol = TypeParameterSymbol;
let TypeAliasSymbol = class TypeAliasSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitTypeAliasSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], TypeAliasSymbol.prototype, "declarations", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypeAliasSymbol.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], TypeAliasSymbol.prototype, "escapedName", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], TypeAliasSymbol.prototype, "name", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Map)
], TypeAliasSymbol.prototype, "members", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], TypeAliasSymbol.prototype, "numberIndexInfo", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Object)
], TypeAliasSymbol.prototype, "stringIndexInfo", void 0);
tslib_1.__decorate([
    core_1.PlainPro({
        isClass: true
    }),
    tslib_1.__metadata("design:type", Object)
], TypeAliasSymbol.prototype, "symbol", void 0);
TypeAliasSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.TypeAlias }
    })
], TypeAliasSymbol);
exports.TypeAliasSymbol = TypeAliasSymbol;
let ExportValueSymbol = class ExportValueSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitExportValueSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ExportValueSymbol.prototype, "flags", void 0);
ExportValueSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.ExportValue }
    })
], ExportValueSymbol);
exports.ExportValueSymbol = ExportValueSymbol;
let AliasSymbol = class AliasSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitAliasSymbol(this, context);
    }
    get(checker) {
        return checker.getDeclaredTypeOfSymbol(this.__node);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], AliasSymbol.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], AliasSymbol.prototype, "declarations", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], AliasSymbol.prototype, "escapedName", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], AliasSymbol.prototype, "name", void 0);
AliasSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Alias }
    })
], AliasSymbol);
exports.AliasSymbol = AliasSymbol;
let PrototypeSymbol = class PrototypeSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitPrototypeSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], PrototypeSymbol.prototype, "flags", void 0);
PrototypeSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Prototype }
    })
], PrototypeSymbol);
exports.PrototypeSymbol = PrototypeSymbol;
let OptionalSymbol = class OptionalSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitOptionalSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], OptionalSymbol.prototype, "flags", void 0);
OptionalSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Optional }
    })
], OptionalSymbol);
exports.OptionalSymbol = OptionalSymbol;
let TransientSymbol = class TransientSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitTransientSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TransientSymbol.prototype, "flags", void 0);
TransientSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Transient }
    })
], TransientSymbol);
exports.TransientSymbol = TransientSymbol;
let AssignmentSymbol = class AssignmentSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitAssignmentSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], AssignmentSymbol.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", String)
], AssignmentSymbol.prototype, "intrinsicName", void 0);
AssignmentSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Assignment }
    })
], AssignmentSymbol);
exports.AssignmentSymbol = AssignmentSymbol;
let ModuleExportsSymbol = class ModuleExportsSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitModuleExportsSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ModuleExportsSymbol.prototype, "flags", void 0);
ModuleExportsSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.ModuleExports }
    })
], ModuleExportsSymbol);
exports.ModuleExportsSymbol = ModuleExportsSymbol;
let EnumSymbol = class EnumSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitEnumSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], EnumSymbol.prototype, "flags", void 0);
EnumSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Enum }
    })
], EnumSymbol);
exports.EnumSymbol = EnumSymbol;
let VariableSymbol = class VariableSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitVariableSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], VariableSymbol.prototype, "flags", void 0);
VariableSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Variable }
    })
], VariableSymbol);
exports.VariableSymbol = VariableSymbol;
let ValueSymbol = class ValueSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitValueSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ValueSymbol.prototype, "flags", void 0);
ValueSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Value }
    })
], ValueSymbol);
exports.ValueSymbol = ValueSymbol;
let TypeSymbol = class TypeSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitTypeSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypeSymbol.prototype, "flags", void 0);
TypeSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Type }
    })
], TypeSymbol);
exports.TypeSymbol = TypeSymbol;
let NamespaceSymbol = class NamespaceSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitNamespaceSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], NamespaceSymbol.prototype, "flags", void 0);
NamespaceSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Namespace }
    })
], NamespaceSymbol);
exports.NamespaceSymbol = NamespaceSymbol;
let ModuleSymbol = class ModuleSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitModuleSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ModuleSymbol.prototype, "flags", void 0);
ModuleSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Module }
    })
], ModuleSymbol);
exports.ModuleSymbol = ModuleSymbol;
let AccessorSymbol = class AccessorSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitAccessorSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], AccessorSymbol.prototype, "flags", void 0);
AccessorSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.Accessor }
    })
], AccessorSymbol);
exports.AccessorSymbol = AccessorSymbol;
let FunctionScopedVariableExcludesSymbol = class FunctionScopedVariableExcludesSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitFunctionScopedVariableExcludesSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], FunctionScopedVariableExcludesSymbol.prototype, "flags", void 0);
FunctionScopedVariableExcludesSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.FunctionScopedVariableExcludes }
    })
], FunctionScopedVariableExcludesSymbol);
exports.FunctionScopedVariableExcludesSymbol = FunctionScopedVariableExcludesSymbol;
let BlockScopedVariableExcludesSymbol = class BlockScopedVariableExcludesSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitBlockScopedVariableExcludesSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], BlockScopedVariableExcludesSymbol.prototype, "flags", void 0);
BlockScopedVariableExcludesSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.BlockScopedVariableExcludes }
    })
], BlockScopedVariableExcludesSymbol);
exports.BlockScopedVariableExcludesSymbol = BlockScopedVariableExcludesSymbol;
let ParameterExcludesSymbol = class ParameterExcludesSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitParameterExcludesSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ParameterExcludesSymbol.prototype, "flags", void 0);
ParameterExcludesSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.ParameterExcludes }
    })
], ParameterExcludesSymbol);
exports.ParameterExcludesSymbol = ParameterExcludesSymbol;
let EnumMemberExcludesSymbol = class EnumMemberExcludesSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitEnumMemberExcludesSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], EnumMemberExcludesSymbol.prototype, "flags", void 0);
EnumMemberExcludesSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.EnumMemberExcludes }
    })
], EnumMemberExcludesSymbol);
exports.EnumMemberExcludesSymbol = EnumMemberExcludesSymbol;
let FunctionExcludesSymbol = class FunctionExcludesSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitFunctionExcludesSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], FunctionExcludesSymbol.prototype, "flags", void 0);
FunctionExcludesSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.FunctionExcludes }
    })
], FunctionExcludesSymbol);
exports.FunctionExcludesSymbol = FunctionExcludesSymbol;
let ClassExcludesSymbol = class ClassExcludesSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitClassExcludesSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ClassExcludesSymbol.prototype, "flags", void 0);
ClassExcludesSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.ClassExcludes }
    })
], ClassExcludesSymbol);
exports.ClassExcludesSymbol = ClassExcludesSymbol;
let InterfaceExcludesSymbol = class InterfaceExcludesSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitInterfaceExcludesSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], InterfaceExcludesSymbol.prototype, "flags", void 0);
InterfaceExcludesSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.InterfaceExcludes }
    })
], InterfaceExcludesSymbol);
exports.InterfaceExcludesSymbol = InterfaceExcludesSymbol;
let RegularEnumExcludesSymbol = class RegularEnumExcludesSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitRegularEnumExcludesSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], RegularEnumExcludesSymbol.prototype, "flags", void 0);
RegularEnumExcludesSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.RegularEnumExcludes }
    })
], RegularEnumExcludesSymbol);
exports.RegularEnumExcludesSymbol = RegularEnumExcludesSymbol;
let ConstEnumExcludesSymbol = class ConstEnumExcludesSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitConstEnumExcludesSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ConstEnumExcludesSymbol.prototype, "flags", void 0);
ConstEnumExcludesSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.ConstEnumExcludes }
    })
], ConstEnumExcludesSymbol);
exports.ConstEnumExcludesSymbol = ConstEnumExcludesSymbol;
let ValueModuleExcludesSymbol = class ValueModuleExcludesSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitValueModuleExcludesSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ValueModuleExcludesSymbol.prototype, "flags", void 0);
ValueModuleExcludesSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.ValueModuleExcludes }
    })
], ValueModuleExcludesSymbol);
exports.ValueModuleExcludesSymbol = ValueModuleExcludesSymbol;
let MethodExcludesSymbol = class MethodExcludesSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitMethodExcludesSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], MethodExcludesSymbol.prototype, "flags", void 0);
MethodExcludesSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.MethodExcludes }
    })
], MethodExcludesSymbol);
exports.MethodExcludesSymbol = MethodExcludesSymbol;
let GetAccessorExcludesSymbol = class GetAccessorExcludesSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitGetAccessorExcludesSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], GetAccessorExcludesSymbol.prototype, "flags", void 0);
GetAccessorExcludesSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.GetAccessorExcludes }
    })
], GetAccessorExcludesSymbol);
exports.GetAccessorExcludesSymbol = GetAccessorExcludesSymbol;
let SetAccessorExcludesSymbol = class SetAccessorExcludesSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitSetAccessorExcludesSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], SetAccessorExcludesSymbol.prototype, "flags", void 0);
SetAccessorExcludesSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.SetAccessorExcludes }
    })
], SetAccessorExcludesSymbol);
exports.SetAccessorExcludesSymbol = SetAccessorExcludesSymbol;
let TypeParameterExcludesSymbol = class TypeParameterExcludesSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitTypeParameterExcludesSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypeParameterExcludesSymbol.prototype, "flags", void 0);
TypeParameterExcludesSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.TypeParameterExcludes }
    })
], TypeParameterExcludesSymbol);
exports.TypeParameterExcludesSymbol = TypeParameterExcludesSymbol;
let TypeAliasExcludesSymbol = class TypeAliasExcludesSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitTypeAliasExcludesSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], TypeAliasExcludesSymbol.prototype, "flags", void 0);
TypeAliasExcludesSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.TypeAliasExcludes }
    })
], TypeAliasExcludesSymbol);
exports.TypeAliasExcludesSymbol = TypeAliasExcludesSymbol;
let AliasExcludesSymbol = class AliasExcludesSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitAliasExcludesSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], AliasExcludesSymbol.prototype, "flags", void 0);
AliasExcludesSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.AliasExcludes }
    })
], AliasExcludesSymbol);
exports.AliasExcludesSymbol = AliasExcludesSymbol;
let ModuleMemberSymbol = class ModuleMemberSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitModuleMemberSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ModuleMemberSymbol.prototype, "flags", void 0);
ModuleMemberSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.ModuleMember }
    })
], ModuleMemberSymbol);
exports.ModuleMemberSymbol = ModuleMemberSymbol;
let ExportHasLocalSymbol = class ExportHasLocalSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitExportHasLocalSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ExportHasLocalSymbol.prototype, "flags", void 0);
ExportHasLocalSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.ExportHasLocal }
    })
], ExportHasLocalSymbol);
exports.ExportHasLocalSymbol = ExportHasLocalSymbol;
let BlockScopedSymbol = class BlockScopedSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitBlockScopedSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], BlockScopedSymbol.prototype, "flags", void 0);
BlockScopedSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.BlockScoped }
    })
], BlockScopedSymbol);
exports.BlockScopedSymbol = BlockScopedSymbol;
let PropertyOrAccessorSymbol = class PropertyOrAccessorSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitPropertyOrAccessorSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], PropertyOrAccessorSymbol.prototype, "flags", void 0);
PropertyOrAccessorSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.PropertyOrAccessor }
    })
], PropertyOrAccessorSymbol);
exports.PropertyOrAccessorSymbol = PropertyOrAccessorSymbol;
let ClassMemberSymbol = class ClassMemberSymbol extends Node {
    visit(visitor, context) {
        return visitor.visitClassMemberSymbol(this, context);
    }
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], ClassMemberSymbol.prototype, "flags", void 0);
ClassMemberSymbol = tslib_1.__decorate([
    core_1.Plain({
        desc: { flags: common_1.ts.SymbolFlags.ClassMember }
    })
], ClassMemberSymbol);
exports.ClassMemberSymbol = ClassMemberSymbol;
let Type = class Type {
};
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], Type.prototype, "id", void 0);
tslib_1.__decorate([
    core_1.PlainPro(),
    tslib_1.__metadata("design:type", Number)
], Type.prototype, "flags", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], Type.prototype, "symbol", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], Type.prototype, "pattern", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Object)
], Type.prototype, "aliasSymbol", void 0);
tslib_1.__decorate([
    core_1.PlainPro({ isClass: true }),
    tslib_1.__metadata("design:type", Array)
], Type.prototype, "aliasTypeArguments", void 0);
Type = tslib_1.__decorate([
    core_1.Plain({
        desc: {
            flags: common_1.ts.TypeFlags
        }
    })
], Type);
exports.Type = Type;
class ObjectType {
}
exports.ObjectType = ObjectType;
class TypeReference {
}
exports.TypeReference = TypeReference;
